#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Storages test backend
"""
from eox_core.test_utils import TestStorage


def get_edxapp_production_staticfiles_storage():  # pylint: disable=invalid-name
    """
    Return the edx-platform production staticfiles storage
    """
    try:
        from openedx.core.storage import ProductionStorage  # pylint: disable=import-outside-toplevel
    except ImportError:
        ProductionStorage = TestStorage
    return ProductionStorage


def get_edxapp_development_staticfiles_storage():  # pylint: disable=invalid-name
    """
    Return the edx-platform development staticfiles storage
    """
    try:
        from openedx.core.storage import DevelopmentStorage  # pylint: disable=import-outside-toplevel
    except ImportError:
        DevelopmentStorage = TestStorage
    return DevelopmentStorage



# -*- coding: utf-8 -*-
""" Tests for public user creation API. """
from __future__ import absolute_import, unicode_literals

from django.test import TestCase
from rest_framework.test import APIRequestFactory, force_authenticate

import eox_core
from eox_core.api.v1.views import UserInfo

from ..test_utils import SuperUserFactory

JSON_CONTENT_TYPE = 'application/json'


class TestInfoView(TestCase):
    """ Tests for the eox-info page """

    def test_version_is_present(self):
        """ Check that test version is present """
        response = self.client.get('/eox-info')
        self.assertContains(response, eox_core.__version__)

    def test_userinfo_endpoint(self):
        """ Tests for /userinfo/ """
        factory = APIRequestFactory()
        view = UserInfo.as_view()
        request = factory.get('/api/v1/userinfo', content_type='application/json')
        user = SuperUserFactory()
        force_authenticate(request, user=user)
        response = view(request)
        response_json = response.data
        self.assertIn('user', response_json)
        self.assertDictEqual(
            response_json, {
                'is_superuser': True,
                'is_staff': True,
                'user': 'robot0',
                'auth': 'None',
                'email': 'robot+test+0@example.com'
            }
        )



"""
Test backend to get CourseEnrollment Model.
"""

from unittest.mock import Mock

from django.contrib.auth.models import Permission

USERNAME_MAX_LENGTH = 30


def get_edxapp_user(**kwargs):
    """
    Return a fake user
    """
    return Mock()


def create_edxapp_user(*args, **kwargs):
    """
    Return a fake user and a list of errors
    """
    return Mock(), []


def get_user_read_only_serializer():
    """
    Return a fake user read only serializer
    """
    try:
        from openedx.core.djangoapps.user_api.accounts.serializers import \
            UserReadOnlySerializer  # pylint: disable=import-outside-toplevel
    except ImportError:
        UserReadOnlySerializer = object
    return UserReadOnlySerializer


def check_edxapp_account_conflicts(email, username):
    """
    Get an executes the check_account_exists for tests
    """
    try:
        from openedx.core.djangoapps.user_api.accounts.api import \
            check_account_exists  # pylint: disable=import-outside-toplevel
    except ImportError:
        def check_account_exists(email=None, username=None):
            """
            Fake definition for check_account_exists edxapp function
            """
            return email and username
    return check_account_exists(email=email, username=username)


def get_course_enrollment():
    """
    Get Test CourseEnrollment model.

    We return any django model that already exists so that
    django-filters is happy and no migrations are created.
    """
    return Permission


def get_course_team_user(*args, **kwargs):
    """
    Return a fake course team user
    """
    return Mock()


def get_user_signup_source():
    """
    Get test UserSignupSource model
    """
    try:
        from common.djangoapps.student.models import UserSignupSource  # pylint: disable=import-outside-toplevel
    except ImportError:
        UserSignupSource = object
    return UserSignupSource


def get_user_profile():
    """ Gets the UserProfile model """
    try:
        from common.djangoapps.student.models import UserProfile  # pylint: disable=import-outside-toplevel
    except ImportError:
        UserProfile = object
    return UserProfile


def generate_password(*args, **kwargs):
    """ Generates a password """
    return "ThisShouldBeARandomPassword"


def get_user_attribute():
    """
    Get test UserAttribute model
    """
    try:
        from common.djangoapps.student.models import UserAttribute  # pylint: disable=import-outside-toplevel
    except ImportError:
        UserAttribute = object
    return UserAttribute



"""
Controllers for the data-api. Used in the report generation process
"""
import random
from datetime import datetime

import six
from django.conf import settings
from django.contrib.auth.models import User
from django.db.models import Q
from django.urls import reverse
from django_filters import rest_framework as filters  # pylint: disable=import-error
from edx_proctoring.models import ProctoredExamStudentAttempt  # pylint: disable=import-error
from rest_framework import mixins, status, viewsets
from rest_framework.authentication import SessionAuthentication
from rest_framework.permissions import IsAdminUser
from rest_framework.response import Response

from eox_core.edxapp_wrapper.bearer_authentication import BearerAuthentication
from eox_core.edxapp_wrapper.certificates import get_generated_certificate
from eox_core.edxapp_wrapper.users import get_course_enrollment

from .filters import CourseEnrollmentFilter, GeneratedCerticatesFilter, ProctoredExamStudentAttemptFilter, UserFilter
from .paginators import DataApiResultsSetPagination
from .serializers import (
    CertificateSerializer,
    CourseEnrollmentSerializer,
    ProctoredExamStudentAttemptSerializer,
    UserSerializer,
)
from .tasks import EnrollmentsGrades


class DataApiViewSet(mixins.ListModelMixin,
                     viewsets.GenericViewSet):
    """
    A generic viewset for all the instances of the data-api
    """
    authentication_classes = (BearerAuthentication, SessionAuthentication)
    permission_classes = (IsAdminUser,)

    pagination_class = DataApiResultsSetPagination
    filter_backends = (filters.DjangoFilterBackend,)
    prefetch_fields = False
    # Microsite enforcement filter settings
    enforce_microsite_filter = False
    enforce_microsite_filter_lookup_field = "test_lookup_field"
    enforce_microsite_filter_term = "org_in_course_id"

    def get_queryset(self):
        """
        This method returns the queryset to be processed by the viewset
        """
        queryset = self.queryset
        if self.prefetch_fields:
            queryset = self.add_prefetch_fields_to_queryset(queryset, self.prefetch_fields)
        if self.enforce_microsite_filter:
            queryset = self.enforce_microsite_filter_qset(queryset)
        return queryset

    def add_prefetch_fields_to_queryset(self, queryset, fields=None):
        """
        This method adds prefetched fields to the queryset in order to
        reduce the number of hits to the database
        """
        if not fields:
            fields = []

        for val in fields:
            if val.get("type", "") == "prefetch":
                queryset = queryset.prefetch_related(val.get("name", ""))
            else:
                queryset = queryset.select_related(val.get("name", ""))

        return queryset

    def enforce_microsite_filter_qset(self, queryset):
        """
        This method enforces the filtering of the queryset based on the
        organization filters belonging to the queried site (which should map
        to a microsite).
        """
        # Check if multitenancy is enabled
        if not settings.EOX_CORE_USER_ENABLE_MULTI_TENANCY:
            return queryset

        orgs_filter = getattr(settings, 'course_org_filter', set([]))

        queryset = self.filter_queryset_by_orgs(
            queryset,
            orgs_filter
        )
        return queryset

    def filter_queryset_by_orgs(self, queryset, org_filters):
        """
        This method filters a given queryset based on the org filters belonging
        to a microsite.
        """
        if not org_filters:
            return queryset.none()

        term_types = {
            "org_exact": "{}",
            "org_in_course_id": ":{}+"
        }
        term = term_types.get(self.enforce_microsite_filter_term, "{}")

        # Handling the case when the course_org_filter value is a string
        if isinstance(org_filters, six.string_types):
            term_search = term.format(org_filters)
            kwargs_filter = {
                self.enforce_microsite_filter_lookup_field: term_search
            }
            queryset = queryset.filter(**kwargs_filter)
            return queryset

        # Handling the case when the course_org_filter value is a list
        query = Q()
        for org in org_filters:
            term_search = term.format(org)
            kwargs_filter = {
                self.enforce_microsite_filter_lookup_field: term_search
            }
            query = query | Q(**kwargs_filter)

        queryset = queryset.filter(query)
        return queryset


class UsersViewSet(DataApiViewSet):  # pylint: disable=too-many-ancestors
    """
    A viewset for viewing users in the platform.
    """
    serializer_class = UserSerializer
    queryset = User.objects.all()
    filter_class = UserFilter
    prefetch_fields = [
        {
            "name": "profile",
            "type": "select"
        },
        {
            "name": "usersignupsource_set",
            "type": "prefetch"
        }
    ]


class CourseEnrollmentViewset(DataApiViewSet):  # pylint: disable=too-many-ancestors
    """
    A viewset for viewing Course Enrollments.
    """
    serializer_class = CourseEnrollmentSerializer
    queryset = get_course_enrollment().objects.all()
    filter_class = CourseEnrollmentFilter
    # Microsite enforcement filter settings
    enforce_microsite_filter = True
    enforce_microsite_filter_lookup_field = "course__id__contains"
    enforce_microsite_filter_term = "org_in_course_id"


class CourseEnrollmentWithGradesViewset(DataApiViewSet):  # pylint: disable=too-many-ancestors
    """
    A viewset for viewing Course Enrollments with grades data.
    This view will create a celery task to fetch grades data for
    enrollments in the background, and will return the id of the
    celery task
    """
    serializer_class = CourseEnrollmentSerializer
    queryset = get_course_enrollment().objects.all()
    filter_class = CourseEnrollmentFilter
    # Microsite enforcement filter settings
    enforce_microsite_filter = True
    enforce_microsite_filter_lookup_field = "course__id__contains"
    enforce_microsite_filter_term = "org_in_course_id"

    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        serializer = self.get_serializer(queryset, many=True)

        now_date = datetime.now()
        string_now_date = now_date.strftime("%Y-%m-%d-%H-%M-%S")
        randnum = random.randint(100, 999)
        task_id = "data_api-" + string_now_date + "-" + str(randnum)

        named_args = {
            "data": serializer.data,
        }

        task = EnrollmentsGrades().apply_async(
            kwargs=named_args,
            task_id=task_id,
            routing_key=settings.GRADES_DOWNLOAD_ROUTING_KEY
        )

        url_task_status = request.build_absolute_uri(
            reverse("eox-core:eox-data-api:celery-data-api-tasks", kwargs={"task_id": task_id})
        )
        data_response = {
            "task_id": task.id,
            "task_url": url_task_status,
        }
        return Response(data_response, status=status.HTTP_202_ACCEPTED)


class CertificateViewSet(DataApiViewSet):  # pylint: disable=too-many-ancestors
    """
    A viewset for viewing certificates generated for users.
    """
    serializer_class = CertificateSerializer
    filter_class = GeneratedCerticatesFilter
    prefetch_fields = [
        {
            "name": "user",
            "type": "select"
        }
    ]
    # Microsite enforcement filter settings
    enforce_microsite_filter = True
    enforce_microsite_filter_lookup_field = "course_id__contains"
    enforce_microsite_filter_term = "org_in_course_id"

    def get_queryset(self):
        self.queryset = get_generated_certificate().objects.all()
        return super().get_queryset()


class ProctoredExamStudentViewSet(DataApiViewSet):  # pylint: disable=too-many-ancestors
    """
    A viewset for viewing proctored exams attempts made by students.
    """

    serializer_class = ProctoredExamStudentAttemptSerializer
    queryset = ProctoredExamStudentAttempt.objects.all()
    filter_class = ProctoredExamStudentAttemptFilter
    prefetch_fields = [
        {
            "name": "user",
            "type": "select"
        },
        {
            "name": "proctored_exam",
            "type": "select"
        }
    ]
    # Microsite enforcement filter settings
    enforce_microsite_filter = True
    enforce_microsite_filter_lookup_field = "proctored_exam__course_id__contains"
    enforce_microsite_filter_term = "org_in_course_id"



#!/usr/bin/python
"""
Test module for Utils
"""
from django.contrib.sites.models import Site
from django.test import TestCase
from mock import patch

from eox_core.utils import fasthash, get_domain_from_oauth_app_uris, get_or_create_site_from_oauth_app_uris


class UtilsTest(TestCase):
    """
    Test the functions included on utils module
    """

    def setUp(self):
        """
        setup.
        """
        super().setUp()
        self.domain_1 = 'test.academy.edunext.io'
        self.domain_2 = 'test.bragi.edunext.io'
        self.site_1 = Site.objects.create(
            domain=self.domain_1,
            name=self.domain_1,
        )
        self.site_2 = Site.objects.create(
            domain=self.domain_2,
            name="Bragi Testing Site",
        )
        self.redirect_uris_http = "http://bragi.localhost:18000/  http://bragi.localhost:18000"
        self.redirect_uris_https = "https://classroomusb.edunext.co https://classroomusb.edunext.co/"

    def test_fasthash_call(self):
        """
        Answers the question: Can we apply the fasthash method?
        """
        test_str = "test_str"
        fasthash(test_str)

    def test_get_domain_from_oauth_app_uris_http_protocol(self):
        """Tests the get_domain_from_oauth_app_uris function
        when the redirect_uris have URLs with http protocol.

        Expected behavior:
            - Returns de domain.
        """
        domain = get_domain_from_oauth_app_uris(self.redirect_uris_http)

        self.assertEqual('bragi.localhost', domain)

    def test_get_domain_from_oauth_app_uris_https_protocol(self):
        """Tests the get_domain_from_oauth_app_uris function
        when the redirect_uris have URLs with https protocol.

        Expected behavior:
            - Returns de domain.
        """
        domain = get_domain_from_oauth_app_uris(self.redirect_uris_https)

        self.assertEqual('classroomusb.edunext.co', domain)

    @patch("eox_core.utils.get_domain_from_oauth_app_uris")
    def test_get_or_create_site_from_oauth_app_uris_existing_site(self, mock_get_domain):
        """Tests the get_or_create_site_from_oauth_app_uris function
        when the redirect_uris are from an existing Django site.

        Expected behavior:
            - Returns de existing site.
        """
        mock_get_domain.return_value = self.domain_1

        site = get_or_create_site_from_oauth_app_uris(self.redirect_uris_https)

        self.assertEqual(self.domain_1, site.domain)
        self.assertEqual(1, Site.objects.filter(domain=self.domain_1).count())
        mock_get_domain.assert_called_once()

    @patch("eox_core.utils.get_domain_from_oauth_app_uris")
    def test_get_or_create_site_from_oauth_app_uris_existing_site_with_different_name(
        self,
        mock_get_domain,
    ):
        """Tests the get_or_create_site_from_oauth_app_uris function
        when the redirect_uris are from an existing Django site.

        Expected behavior:
            - Returns de existing site.
        """
        mock_get_domain.return_value = self.domain_2

        site = get_or_create_site_from_oauth_app_uris(self.redirect_uris_https)

        self.assertEqual(self.domain_2, site.domain)
        self.assertEqual(1, Site.objects.filter(domain=self.domain_2).count())
        mock_get_domain.assert_called_once()

    @patch("eox_core.utils.get_domain_from_oauth_app_uris")
    def test_get_or_create_site_from_oauth_app_uris_new_site(
        self,
        mock_get_domain,
    ):
        """Tests the get_or_create_site_from_oauth_app_uris function
        when the redirect_uris don't have a Django Site.

        Expected behavior:
            - Creates and returns the new site.
        """
        new_site_domain = "new-site.edunext.io"
        mock_get_domain.return_value = new_site_domain

        site = get_or_create_site_from_oauth_app_uris(self.redirect_uris_https)

        self.assertEqual(new_site_domain, site.domain)
        self.assertEqual(1, Site.objects.filter(domain=self.domain_1).count())
        mock_get_domain.assert_called_once()



"""
Integration test suite.

This suite performs multiple http requests to guarantee
that all CRUD operations are handled correctly.
"""
import json
from os import environ

import pytest
import requests
from django.test import TestCase


@pytest.mark.skipif(not environ.get("TEST_INTEGRATION"), reason="Run only explicitly")
class TestEnrollmentIntegration(TestCase):  # pragma: no cover
    # pylint: disable=too-many-public-methods
    """Test suite"""
    data = {}

    @classmethod
    def setUpClass(cls):
        with open("eox_core/tests/integration/test_data", encoding="utf-8") as file_obj:
            cls.data = json.load(file_obj)
        cls.data["endpoint"] = "eox-core/api/v1/enrollment/"
        site1_data = {
            "client_id": cls.data["site1_data"]["client_id"],
            "client_secret": cls.data["site1_data"]["client_secret"],
            "grant_type": "client_credentials",
        }
        site2_data = {
            "client_id": cls.data["site2_data"]["client_id"],
            "client_secret": cls.data["site2_data"]["client_secret"],
            "grant_type": "client_credentials",
        }
        request_url = f"{cls.data['site1_data']['base_url']}/oauth2/access_token/"
        response_site1 = requests.post(request_url, data=site1_data, timeout=10)
        response_site1.raise_for_status()
        cls.data["site1_data"]["token"] = response_site1.json()["access_token"]
        request_url = f"{cls.data['site2_data']['base_url']}/oauth2/access_token/"
        response_site2 = requests.post(request_url, data=site2_data, timeout=10)
        response_site2.raise_for_status()
        cls.data["site2_data"]["token"] = response_site2.json()["access_token"]

    @classmethod
    def tearDownClass(cls):
        delete_enrollment(cls.data)

    def test_read_valid_email_course(self):
        # pylint: disable=invalid-name
        """
        Get a valid enrollment
        """
        create_enrollment(self.data)
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        expected_response = {
            "username": site1_data["user_id"],
            "course_id": data["course_id"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.get(request_url, data=data, headers=headers, timeout=10)
        response_content = response.json()

        self.assertEqual(response.status_code, 200)
        self.assertDictContainsSubset(expected_response, response_content)

    def test_read_invalid_enrollment(self):
        # pylint: disable=invalid-name
        """
        Get a invalid enrollment (doesn't exist)
        """
        delete_enrollment(self.data)
        site1_data = self.data["site1_data"]
        data = {
            "username": site1_data["user_id"],
            "course_id": site1_data["course"]["id"],
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.get(request_url, data=data, headers=headers, timeout=10)

        self.assertEqual(response.status_code, 404)

    def test_read_invalid_enrollment_for_site(self):
        # pylint: disable=invalid-name
        """
        Get a invalid enrollment (enrollment from other site)
        """
        create_enrollment(self.data)
        site1_data = self.data["site1_data"]
        site2_data = self.data["site2_data"]
        data = {
            "username": site1_data["user_id"],
            "course_id": site1_data["course"]["id"],
        }
        headers = {
            "Authorization": f"Bearer {site2_data['token']}",
            "Host": site2_data["host"],
        }
        request_url = f"{site2_data['base_url']}/{self.data['endpoint']}"

        response = requests.get(request_url, data=data, headers=headers, timeout=10)

        self.assertEqual(response.status_code, 404)

    def test_create_enrollment_valid_user_mode_course(self):
        # pylint: disable=invalid-name
        """
        Create enrollment with a valid user, valid course,
        valid mode
        """
        delete_enrollment(self.data)
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
            "mode": site1_data["course"]["mode"],
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.post(request_url, data=data, headers=headers, timeout=10)

        self.assertEqual(response.status_code, 200)

    def test_force_create_enrollment_valid_user_mode_course(self):
        # pylint: disable=invalid-name
        """
        Create enrollment with a valid user, valid course,
        valid mode using force
        """
        delete_enrollment(self.data)
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
            "mode": site1_data["course"]["mode"],
            "force": True,
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.post(request_url, data=data, headers=headers, timeout=10)

        self.assertEqual(response.status_code, 200)

    def test_create_valid_course_mode_invalid_user(self):
        # pylint: disable=invalid-name
        """
        Create enrollment with a valid course, valid mode,
        and a non-existent user
        """
        site1_data = self.data["site1_data"]
        data = {
            "username": site1_data["fake_user"],
            "course_id": site1_data["course"]["id"],
            "mode": site1_data["course"]["mode"],
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.post(request_url, data=data, headers=headers, timeout=10)

        self.assertEqual(response.status_code, 400)

    def test_create_valid_course_mode_invalid_user_for_site(self):
        # pylint: disable=invalid-name
        """
        Create enrollment with a valid course, valid mode,
        and a user from another site
        """
        site1_data = self.data["site1_data"]
        site2_data = self.data["site2_data"]
        data = {
            "email": site2_data["user_email"],
            "course_id": site1_data["course"]["id"],
            "mode": site1_data["course"]["mode"],
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.post(request_url, data=data, headers=headers, timeout=10)

        self.assertEqual(response.status_code, 202)

    def test_create_valid_user_mode_invalid_course(self):
        # pylint: disable=invalid-name
        """
        Create enrollment with a valid user, valid mode,
        and non-existent course
        """
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": "fake_course_id",
            "mode": "audit",
            "force": True,
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.post(request_url, data=data, headers=headers, timeout=10)

        self.assertEqual(response.status_code, 400)

    def test_create_valid_user_mode_invalid_course_for_site(self):
        # pylint: disable=invalid-name
        """
        Create enrollment with a valid user, valid mode,
        and a course from another site
        """
        site1_data = self.data["site1_data"]
        site2_data = self.data["site2_data"]
        data = {
            "email": site2_data["user_email"],
            "course_id": site1_data["course"]["id"],
            "mode": site1_data["course"]["mode"],
            "force": True,
        }
        headers = {
            "Authorization": f"Bearer {site2_data['token']}",
            "Host": site2_data["host"],
        }
        request_url = f"{site2_data['base_url']}/{self.data['endpoint']}"

        response = requests.post(request_url, data=data, headers=headers, timeout=10)

        self.assertEqual(response.status_code, 400)

    # NOTE: Mode changes are not working correctly on devstack
    def test_force_create_valid_user_course_invalid_mode(self):
        # pylint: disable=invalid-name
        """
        Create enrollment with a valid user, valid course,
        and a not available mode
        """
        delete_enrollment(self.data)
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
            "mode": "masters",
            "force": 1,
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.post(request_url, data=data, headers=headers, timeout=10)

        self.assertEqual(response.status_code, 400)

    def test_delete_valid_enrollment(self):
        # pylint: disable=invalid-name
        """
        Delete a valid enrollment
        """
        create_enrollment(self.data)
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.delete(request_url, data=data, headers=headers, timeout=10)

        self.assertEqual(response.status_code, 204)

    def test_delete_invalid_enrollment(self):
        # pylint: disable=invalid-name
        """
        Delete a invalid enrollment(doesn't exist)
        """
        delete_enrollment(self.data)
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.delete(request_url, data=data, headers=headers, timeout=10)

        self.assertEqual(response.status_code, 404)

    def test_delete_invalid_enrollment_for_site(self):
        # pylint: disable=invalid-name
        """
        Delete a invalid enrollment (enrollment from other site)
        """
        create_enrollment(self.data)
        site1_data = self.data["site1_data"]
        site2_data = self.data["site2_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
        }
        headers = {
            "Authorization": f"Bearer {site2_data['token']}",
            "Host": site2_data["host"],
        }
        request_url = f"{site2_data['base_url']}/{self.data['endpoint']}"

        response = requests.delete(request_url, data=data, headers=headers, timeout=10)

        self.assertEqual(response.status_code, 404)

    def test_update_valid_enrollment_change_is_active(self):
        # pylint: disable=invalid-name
        """
        Update an existing enrollment; change is_active flag
        """
        create_enrollment(self.data)
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
            "is_active": False,
            "mode": site1_data["course"]["mode"],
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        expected_response = {
            "user": site1_data["user_id"],
            "is_active": False,
            "course_id": data["course_id"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.put(request_url, data=data, headers=headers, timeout=10)
        response_content = response.json()

        self.assertEqual(response.status_code, 200)
        self.assertDictContainsSubset(expected_response, response_content)

    # NOTE: Mode changes are not working correctly on devstack
    def test_update_valid_enrollment_change_valid_mode(self):
        # pylint: disable=invalid-name
        """
        Update an existing enrollment; change mode
        """
        create_enrollment(self.data)
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
            "is_active": True,
            "mode": "honor",
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        expected_response = {
            "user": site1_data["user_id"],
            "is_active": True,
            "course_id": data["course_id"],
            "mode": "honor",
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.put(request_url, data=data, headers=headers, timeout=10)
        response_content = response.json()

        self.assertEqual(response.status_code, 200)
        self.assertDictContainsSubset(expected_response, response_content)

    def test_update_valid_enrollment_change_invalid_mode(self):
        # pylint: disable=invalid-name
        """
        Update an existing enrollment; change to invalid mode
        """
        create_enrollment(self.data)
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
            "is_active": True,
            "mode": "masters",
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.put(request_url, data=data, headers=headers, timeout=10)

        self.assertEqual(response.status_code, 400)

    def test_update_invalid_enrollment_change_valid_mode(self):
        # pylint: disable=invalid-name
        """
        Update an non-existent enrollment; change mode
        """
        delete_enrollment(self.data)
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
            "is_active": True,
            "mode": "honor",
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.put(request_url, data=data, headers=headers, timeout=10)

        self.assertEqual(response.status_code, 202)

    def test_update_invalid_enrollment_change_is_active(self):
        # pylint: disable=invalid-name
        """
        Update an non-existent enrollment; change is_active flag
        """
        delete_enrollment(self.data)
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
            "is_active": False,
            "mode": "audit",
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.put(request_url, data=data, headers=headers, timeout=10)

        self.assertEqual(response.status_code, 202)

    def test_update_valid_enrollment_change_is_active_force_post(self):
        # pylint: disable=invalid-name
        """
        Update an existing enrollment using POST with force=True;
        change is_active flag
        """
        create_enrollment(self.data)
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
            "is_active": False,
            "mode": site1_data["course"]["mode"],
            "force": True,
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        expected_response = {
            "username": site1_data["user_id"],
            "is_active": False,
            "course_id": data["course_id"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.post(request_url, data=data, headers=headers, timeout=10)
        response_content = response.json()

        self.assertEqual(response.status_code, 200)
        self.assertDictContainsSubset(expected_response, response_content)

    # NOTE: Mode changes are not working correctly on devstack
    def test_update_valid_enrollment_change_valid_mode_force_post(self):
        # pylint: disable=invalid-name
        """
        Update an existing enrollment; change mode
        """
        create_enrollment(self.data)
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
            "is_active": True,
            "mode": "honor",
            "force": True,
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        expected_response = {
            "user": site1_data["user_id"],
            "is_active": True,
            "course_id": data["course_id"],
            "mode": "honor",
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.put(request_url, data=data, headers=headers, timeout=10)
        response_content = response.json()

        self.assertEqual(response.status_code, 200)
        self.assertDictContainsSubset(expected_response, response_content)


def create_enrollment(data):
    """
    Auxiliary function to setUp test fixtures. Creates/enables a new enrollment.

    :param data: dictionary with all the parameters needed to create an enrollment.
    """
    req_data = {
        "email": data["site1_data"]["user_email"],
        "course_id": data["site1_data"]["course"]["id"],
        "mode": data["site1_data"]["course"]["mode"],
    }
    headers = {
        "Authorization": f"Bearer {data['site1_data']['token']}",
        "Host": data["site1_data"]["host"],
    }
    request_url = f"{data['site1_data']['base_url']}/{data['endpoint']}"
    response = requests.post(request_url, data=req_data, headers=headers, timeout=10)
    response.raise_for_status()


def delete_enrollment(data):
    """
    Auxiliary function to setUp test fixtures. Deletes an enrollment if exists.

    :param data: dictionary with all the parameters needed to delete an enrollment.
    """
    req_data = {
        "email": data["site1_data"]["user_email"],
        "course_id": data["site1_data"]["course"]["id"],
    }
    headers = {
        "Authorization": f"Bearer {data['site1_data']['token']}",
        "Host": data["site1_data"]["host"],
    }
    request_url = f"{data['site1_data']['base_url']}/{data['endpoint']}"
    response = requests.delete(request_url, data=req_data, headers=headers, timeout=10)
    if response.status_code == 404:
        return
    response.raise_for_status()



"""
Functions copied from a version higer than hawthorn (for backwards compatibility with it)
Must be deleted some day and replaced with calls to the actual functions
"""
from django.conf import settings
# pylint: disable=import-error
from django.core.cache import cache
from openedx.core.djangoapps.catalog.models import CatalogIntegration
from openedx.core.djangoapps.catalog.utils import get_catalog_api_client as create_catalog_api_client


def get_program(program_uuid, ignore_cache=False):
    """
    Retrieves the details for the specified program.

     Args:
         program_uuid (UUID): Program identifier
         ignore_cache (bool): Indicates if previously-cached data should be ignored.

     Returns:
         dict
    """
    program_uuid = str(program_uuid)
    cache_key = f'programs.api.data.{program_uuid}'

    if not ignore_cache:
        program = cache.get(cache_key)

        if program:
            return program

    catalog_integration = CatalogIntegration.current()
    user = catalog_integration.get_service_user()
    api = create_catalog_api_client(user)

    program = api.programs(program_uuid).get()
    cache.set(cache_key, program, getattr(settings, 'PROGRAMS_CACHE_TTL', 60))

    return program



"""
TODO: add me
"""
from rest_framework import routers

from .viewsets import (
    CertificateViewSet,
    CourseEnrollmentViewset,
    CourseEnrollmentWithGradesViewset,
    ProctoredExamStudentViewSet,
    UsersViewSet,
)

ROUTER = routers.DefaultRouter()
ROUTER.register(r'users', UsersViewSet)
ROUTER.register(r'course-enrollments', CourseEnrollmentViewset)
ROUTER.register(r'certificates', CertificateViewSet, basename='generated_certificate')
ROUTER.register(r'proctored-exams-attempts', ProctoredExamStudentViewSet)
# Async operations
ROUTER.register(r'async/course-enrollments-grades', CourseEnrollmentWithGradesViewset)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Backend for the create_edxapp_user that works under the open-release/lilac.master tag
"""
import logging

from common.djangoapps.student.helpers import (  # pylint: disable=import-error,no-name-in-module
    create_or_set_user_attribute_created_on_site,
    do_create_account,
)
from common.djangoapps.student.models import (  # pylint: disable=import-error,no-name-in-module
    CourseEnrollment,
    LoginFailures,
    Registration,
    UserAttribute,
    UserProfile,
    UserSignupSource,
    create_comments_service_user,
    email_exists_or_retired,
    get_retired_email_by_email,
    username_exists_or_retired,
)
from crum import get_current_user
from django import forms
from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import transaction
from openedx.core.djangoapps.lang_pref import LANGUAGE_KEY  # pylint: disable=import-error
from openedx.core.djangoapps.site_configuration import helpers as configuration_helpers  # pylint: disable=import-error
from openedx.core.djangoapps.user_api.accounts import USERNAME_MAX_LENGTH  # pylint: disable=import-error,unused-import
from openedx.core.djangoapps.user_api.accounts.serializers import UserReadOnlySerializer  # pylint: disable=import-error
from openedx.core.djangoapps.user_api.accounts.views import \
    _set_unusable_password  # pylint: disable=import-error,unused-import
from openedx.core.djangoapps.user_api.models import UserRetirementStatus  # pylint: disable=import-error
from openedx.core.djangoapps.user_api.preferences import api as preferences_api  # pylint: disable=import-error
from openedx.core.djangoapps.user_authn.utils import generate_password  # pylint: disable=import-error,unused-import
from openedx.core.djangoapps.user_authn.views.register import \
    REGISTER_USER as post_register  # pylint: disable=import-error
from openedx.core.djangoapps.user_authn.views.registration_form import (  # pylint: disable=import-error
    AccountCreationForm,
)
from openedx.core.djangolib.oauth2_retirement_utils import \
    retire_dot_oauth2_models  # pylint: disable=import-error,unused-import
from openedx_events.learning.data import UserData, UserPersonalData  # pylint: disable=import-error
from openedx_events.learning.signals import STUDENT_REGISTRATION_COMPLETED  # pylint: disable=import-error
from rest_framework import status
from rest_framework.exceptions import NotFound
from social_django.models import UserSocialAuth  # pylint: disable=import-error

LOG = logging.getLogger(__name__)
User = get_user_model()  # pylint: disable=invalid-name


def get_user_read_only_serializer():
    """
    Great serializer that fits our needs
    """
    return UserReadOnlySerializer


def check_edxapp_account_conflicts(email, username):
    """
    Exposed function to check conflicts
    """
    conflicts = []

    if username and username_exists_or_retired(username):
        conflicts.append("username")

    if email and email_exists_or_retired(email):
        conflicts.append("email")

    return conflicts


def is_allowed_to_skip_extra_registration_fields(account_creation_data):
    """
    Returns True if the conditions are met to skip sending the extra
    registrations fields durint the account creation.

    Three conditions are checked in order to be able to skip
    these registration fields:
    1. The skip_extra_registration_fields field is sent in the data
        for the account creation as True.
    2. The user making the request is staff.
    3. The data received in the parameters does not contain any of the
        REGISTRATION_EXTRA_FIELDS configured for the microsite.

    In case any of these conditions is not met the function returns
    False.
    """
    skip_extra_registration_fields = account_creation_data.pop("skip_extra_registration_fields", False)
    current_user = get_current_user()
    extra_fields = getattr(settings, "REGISTRATION_EXTRA_FIELDS", {})
    extended_profile_fields = getattr(settings, "extended_profile_fields", [])

    if not (skip_extra_registration_fields and current_user.is_staff):
        return False

    for field in account_creation_data.keys():
        if field in extra_fields.keys() or field in extended_profile_fields:
            return False

    return True


class EdnxAccountCreationForm(AccountCreationForm):
    """
    A form to extend the behaviour of the AccountCreationForm.
    For now the purpose of this form is to allow to make the
    password optional if the flag 'skip_password' is True.
    This form it's currently only used for validation, not rendering.
    """

    def __init__(  # pylint:disable=too-many-arguments
            self,
            data=None,
            extra_fields=None,
            extended_profile_fields=None,
            do_third_party_auth=True,
            tos_required=True,
    ):
        super().__init__(
            data=data,
            extra_fields=extra_fields,
            extended_profile_fields=extended_profile_fields,
            do_third_party_auth=do_third_party_auth,
            tos_required=tos_required,
        )

        if data.pop("skip_password", False):
            self.fields['password'] = forms.CharField(required=False)


def create_edxapp_user(*args, **kwargs):
    """
    Creates a user on the open edx django site using calls to
    functions defined in the edx-platform codebase

    Example call:

    data = {
        'email': "address@example.org",
        'username': "Username",
        'password': "P4ssW0rd",
        'fullname': "Full Name",
        'activate': True,
        'site': request.site,
        'language_preference': 'es-419',
    }
    user = create_edxapp_user(**data)

    """
    errors = []

    kwargs["name"] = kwargs.pop("fullname", None)
    email = kwargs.get("email")
    username = kwargs.get("username")
    conflicts = check_edxapp_account_conflicts(email=email, username=username)
    if conflicts:
        return None, [f"Fatal: account collition with the provided: {', '.join(conflicts)}"]

    account_creation_form_data = {
        "data": kwargs,
        "tos_required": False,
        # "enforce_password_policy": enforce_password_policy,
    }

    # Check if we should send the extra registration fields
    if not is_allowed_to_skip_extra_registration_fields(kwargs):
        account_creation_form_data["extra_fields"] = getattr(settings, "REGISTRATION_EXTRA_FIELDS", {})
        account_creation_form_data["extended_profile_fields"] = getattr(settings, "extended_profile_fields", [])

    # Go ahead and create the new user
    with transaction.atomic():
        # In theory is possible to extend the registration form with a custom app
        # An example form app for this can be found at http://github.com/open-craft/custom-form-app
        # form = get_registration_extension_form(data=params)
        # if not form:
        form = EdnxAccountCreationForm(**account_creation_form_data)
        (user, profile, registration) = do_create_account(form)  # pylint: disable=unused-variable

    site = kwargs.pop("site", False)
    if site:
        create_or_set_user_attribute_created_on_site(user, site)
    else:
        errors.append("The user was not assigned to any site")

    try:
        create_comments_service_user(user)
    except Exception:  # pylint: disable=broad-except
        errors.append("No comments_service_user was created")

    # TODO: link account with third party auth

    # Announce registration through API call
    post_register.send_robust(sender=None, user=user)  # pylint: disable=no-member

    STUDENT_REGISTRATION_COMPLETED.send_event(
        user=UserData(
            pii=UserPersonalData(
                username=user.username,
                email=user.email,
                name=user.profile.name,  # pylint: disable=no-member
            ),
            id=user.id,
            is_active=user.is_active,
        ),
    )

    lang_pref = kwargs.pop("language_preference", False)
    if lang_pref:
        try:
            preferences_api.set_user_preference(user, LANGUAGE_KEY, lang_pref)
        except Exception:  # pylint: disable=broad-except
            errors.append(f"Could not set lang preference '{lang_pref}' for user '{user.username}'")

    if kwargs.pop("activate_user", False):
        user.is_active = True
        user.save()

    # TODO: run conditional email sequence

    return user, errors


def get_edxapp_user(**kwargs):
    """
    Retrieve a user by username and/or email

    The user will be returned only if it belongs to the calling site

    Examples:
        >>> get_edxapp_user(
            {
                "username": "Bob",
                "site": request.site
            }
        )
        >>> get_edxapp_user(
            {
                "email": "Bob@mailserver.com",
                "site": request.site
            }
        )
    """
    params = {key: kwargs.get(key) for key in ['username', 'email'] if key in kwargs}
    site = kwargs.get('site')
    try:
        domain = site.domain
    except AttributeError:
        domain = None

    try:
        user = User.objects.get(**params)
        for source_method in FetchUserSiteSources.get_enabled_source_methods():
            if source_method(user, domain):
                break
        else:
            raise User.DoesNotExist
    except User.DoesNotExist:
        raise NotFound(f'No user found by {str(params)} on site {domain}.') from User.DoesNotExist
    return user


def delete_edxapp_user(*args, **kwargs):
    """
    Deletes a user from the platform.
    """
    msg = None

    user = kwargs.get("user")
    case_id = kwargs.get("case_id")
    site = kwargs.get("site")
    is_support_user = kwargs.get("is_support_user")

    user_response = f"The user {user.username} <{user.email}> "

    signup_sources = user.usersignupsource_set.all()
    sources = [signup_source.site for signup_source in signup_sources]

    if site and site.name.upper() in (source.upper() for source in sources):
        if len(sources) == 1:
            with transaction.atomic():
                support_label = "_support" if is_support_user else ""
                user.email = f"{user.email}{case_id}.ednx{support_label}_retired"
                user.save()

                # Add user to retirement queue.
                UserRetirementStatus.create_retirement(user)

                # Unlink LMS social auth accounts
                UserSocialAuth.objects.filter(user_id=user.id).delete()

                # Change LMS password & email
                user.email = get_retired_email_by_email(user.email)
                user.save()
                _set_unusable_password(user)

                # Remove the activation keys sent by email to the user for account activation.
                Registration.objects.filter(user=user).delete()

                # Delete OAuth tokens associated with the user.
                retire_dot_oauth2_models(user)

                # Delete user signup source object
                signup_sources[0].delete()

                msg = f"{user_response} has been removed"
        else:
            for signup_source in signup_sources:
                if signup_source.site.upper() == site.name.upper():
                    signup_source.delete()

                    msg = f"{user_response} has more than one signup source. The signup source from the site {site} has been deleted"

        return msg, status.HTTP_200_OK

    raise NotFound(f"{user_response} does not have a signup source on the site {site}")


def get_course_team_user(*args, **kwargs):
    """
    Get _course_team_user function.
    We need to check if the SERVICE_VARIANT is equal to cms, since
    contentstore is a module registered in the INSTALLED_APPS
    of the cms only.
    """
    if settings.SERVICE_VARIANT == 'cms':
        from contentstore.views.user import _course_team_user  # pylint: disable=import-error,import-outside-toplevel
        return _course_team_user(*args, **kwargs)
    return None


class FetchUserSiteSources:
    """
    Methods to make the comparison to check if an user belongs to a site plus the
    get_enabled_source_methods that just brings an array of functions enabled to do so
    """

    @classmethod
    def get_enabled_source_methods(cls):
        """ Brings the array of methods to check if an user belongs to a site. """
        sources = configuration_helpers.get_value(
            'EOX_CORE_USER_ORIGIN_SITE_SOURCES',
            getattr(settings, 'EOX_CORE_USER_ORIGIN_SITE_SOURCES')
        )
        return [getattr(cls, source) for source in sources]

    @staticmethod
    def fetch_from_created_on_site_prop(user, domain):
        """ Fetch option. """
        if not domain:
            return False
        return UserAttribute.get_user_attribute(user, 'created_on_site') == domain

    @staticmethod
    def fetch_from_user_signup_source(user, domain):
        """ Read the signup source. """
        return len(UserSignupSource.objects.filter(user=user, site=domain)) > 0

    @staticmethod
    def fetch_from_unfiltered_table(user, site):
        """ Fetch option that does not take into account the multi-tentancy model of the installation. """
        return bool(user)


def get_course_enrollment():
    """ get CourseEnrollment model """
    return CourseEnrollment


def get_user_signup_source():
    """ get UserSignupSource model """
    return UserSignupSource


def get_login_failures():
    """ get LoginFailures model """
    return LoginFailures


def get_user_profile():
    """ Gets the UserProfile model """

    return UserProfile


def get_user_attribute():
    """ Gets the UserAttribute model """
    return UserAttribute



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Backend for the CourseKey validations that works under the open-release/maple.master tag
"""
# pylint: disable=import-error, protected-access
from __future__ import absolute_import, unicode_literals

from django.conf import settings
from opaque_keys import InvalidKeyError
from opaque_keys.edx.keys import CourseKey
from openedx.core.djangoapps.site_configuration.helpers import get_all_orgs, get_current_site_orgs
from rest_framework.serializers import ValidationError


def get_valid_course_key(course_id):
    """
    Return the CourseKey if the course_id is valid
    """
    try:
        return CourseKey.from_string(course_id)
    except InvalidKeyError:
        raise ValidationError(f"Invalid course_id {course_id}") from InvalidKeyError


def validate_org(course_id):
    """
    Validate the course organization against all possible orgs for the site

    To determine if the Org is valid we must look at 3 things
    1 Orgs in the current site
    2 Orgs in other sites
    3 flag EOX_CORE_USER_ENABLE_MULTI_TENANCY
    """

    if not settings.EOX_CORE_USER_ENABLE_MULTI_TENANCY:
        return True

    course_key = get_valid_course_key(course_id)
    course_org_filter = getattr(settings, "course_org_filter", [])
    course_org_filter = course_org_filter if isinstance(course_org_filter, list) else [course_org_filter]
    current_site_orgs = get_current_site_orgs() or course_org_filter or []

    if not current_site_orgs:  # pylint: disable=no-else-return
        if course_key.org in get_all_orgs():
            return False
        return True
    else:
        return course_key.org in current_site_orgs



"""
TODO: add me
"""
from django.conf import settings
from rest_framework.pagination import PageNumberPagination


class DataApiResultsSetPagination(PageNumberPagination):
    """
    A subset of data of any queryset
    """
    page_size = settings.DATA_API_DEF_PAGE_SIZE
    page_size_query_param = 'page_size'
    max_page_size = settings.DATA_API_MAX_PAGE_SIZE



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Eox-core storages
"""
from eox_core.edxapp_wrapper.storages import (
    get_edxapp_development_staticfiles_storage,
    get_edxapp_production_staticfiles_storage,
)

EdxappProductionStorage = get_edxapp_production_staticfiles_storage()  # pylint: disable=invalid-name
EdxappDevelopmentStorage = get_edxapp_development_staticfiles_storage()  # pylint: disable=invalid-name


class AbsoluteUrlAssetsMixin:
    """
    Mixin that overrides the url method on storages
    """
    def url(self, name):
        """
        Return url of the asset.
        If the asset name is an absolute url, just return the asset name
        """
        if name.startswith("https://") or name.startswith("http://"):
            return name

        return super().url(name)


class ProductionStorage(AbsoluteUrlAssetsMixin, EdxappProductionStorage):
    """
    Eox-core production extended static files storage
    """


class DevelopmentStorage(AbsoluteUrlAssetsMixin, EdxappDevelopmentStorage):
    """
    Eox-core development extended static files storage
    """



#!/usr/bin/python
"""
Tests the separation layer between edxapp and the plugin
"""
from django.test import TestCase
from mock import Mock, patch

from eox_core.edxapp_wrapper import configuration_helpers


class ConfigurationHelpersTest(TestCase):
    """
    Making sure that the configuration_helpers backend works
    """

    @patch('eox_core.edxapp_wrapper.configuration_helpers.import_module')
    def test_imported_module_is_used(self, import_mock):
        """
        Testing the backend is imported and used
        """
        backend = Mock()
        import_mock.side_effect = backend

        configuration_helpers.get_configuration_helper()

        import_mock.assert_called()
        backend.assert_called()



""" urls.py """

from django.urls import re_path

from eox_core.api.support.v1 import views

app_name = 'eox_core'  # pylint: disable=invalid-name

urlpatterns = [  # pylint: disable=invalid-name
    re_path(r'^user/$', views.EdxappUser.as_view(), name='edxapp-user'),
    re_path(r'^user/replace-username/$', views.EdxappReplaceUsername.as_view(), name='edxapp-replace-username'),
    re_path(r'^oauth-application/$', views.OauthApplicationAPIView.as_view(), name='edxapp-oauth-application'),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Test module for PreEnrollments under the open-release/hawthorn.beta1 tag
"""
from __future__ import absolute_import, unicode_literals

import mock
from django.conf import settings
from django.test import TestCase

from ..pre_enrollments import create_pre_enrollment, delete_pre_enrollment, get_pre_enrollment, update_pre_enrollment


class PreEnrollmentTest(TestCase):
    """ Tests correct load of PreEnrollment module """

    def setUp(self):
        """ setup """
        super().setUp()
        self.m_params = {
            'email': 'test@example.com',
            'course_id': 'course-v1:org+course+run',
            'auto_enroll': True,
        }

    @mock.patch('eox_core.edxapp_wrapper.pre_enrollments.import_module')
    def test_import_the_backend(self, m_import):
        """ Test we import the correct backend defined in the settings """

        create_pre_enrollment()
        m_import.assert_called_with(settings.EOX_CORE_PRE_ENROLLMENT_BACKEND)

    @mock.patch('eox_core.edxapp_wrapper.pre_enrollments.import_module')
    def test_call_the_backend(self, m_import):
        """ Test we use the imported backend """
        m_pre_enrollment_backend = mock.MagicMock()
        m_import.return_value = m_pre_enrollment_backend

        create_pre_enrollment(self.m_params)
        m_pre_enrollment_backend.create_pre_enrollment.assert_called_with(self.m_params)

        update_pre_enrollment(self.m_params)
        m_pre_enrollment_backend.update_pre_enrollment.assert_called_with(self.m_params)

        delete_pre_enrollment(self.m_params)
        m_pre_enrollment_backend.delete_pre_enrollment.assert_called_with(self.m_params)

        get_pre_enrollment(self.m_params)
        m_pre_enrollment_backend.get_pre_enrollment.assert_called_with(self.m_params)



""" Support API urls.py """

from django.urls import include, re_path

app_name = 'eox_core'  # pylint: disable=invalid-name

urlpatterns = [  # pylint: disable=invalid-name
    re_path(r'^v1/', include('eox_core.api.support.v1.urls', namespace='eox-support-api')),
]



"""
TODO: add me
"""
import django_filters  # pylint: disable=import-error
from django.contrib.auth.models import User
from edx_proctoring.models import ProctoredExamStudentAttempt  # pylint: disable=import-error
from opaque_keys.edx.keys import CourseKey  # pylint: disable=import-error

from eox_core.edxapp_wrapper.certificates import get_generated_certificate
from eox_core.edxapp_wrapper.users import get_course_enrollment


class BaseDataApiFilter(django_filters.rest_framework.FilterSet):
    """
    TODO: add me
    """
    ordering = django_filters.OrderingFilter()


class UserFilter(BaseDataApiFilter):
    """
    TODO: add me
    """
    # Filtering by main model fields
    username = django_filters.CharFilter(lookup_expr='icontains')
    first_name = django_filters.CharFilter(lookup_expr='icontains')
    last_name = django_filters.CharFilter(lookup_expr='icontains')
    email = django_filters.CharFilter(lookup_expr='icontains')
    is_active = django_filters.BooleanFilter()
    date_joined = django_filters.DateTimeFromToRangeFilter()

    # Filtering by user profile fields
    name = django_filters.CharFilter(field_name="profile__name", lookup_expr="icontains")
    language = django_filters.CharFilter(field_name="profile__language", lookup_expr="iexact")
    year_of_birth = django_filters.RangeFilter(field_name="profile__year_of_birth")
    gender = django_filters.CharFilter(field_name="profile__gender", lookup_expr="iexact")
    mailing_address = django_filters.CharFilter(field_name="profile__mailing_address", lookup_expr="iexact")
    city = django_filters.CharFilter(field_name="profile__city", lookup_expr="icontains")
    country = django_filters.CharFilter(field_name="profile__country", lookup_expr="icontains")

    # Filtering by user signup source fields
    site = django_filters.CharFilter(field_name="usersignupsource__site", lookup_expr='iexact')

    class Meta:
        """
        TODO: add me
        """
        model = User
        fields = [
            'id',
            'username',
            'first_name',
            'last_name',
            'email',
            'is_active',
            'date_joined',
            'name',
            'language',
            'year_of_birth',
            'gender',
            'mailing_address',
            'city',
            'country',
            'site',
        ]


class CourseEnrollmentFilter(BaseDataApiFilter):
    """
    TODO: add me
    """
    course_id = django_filters.CharFilter(method="filter_course_id")
    created = django_filters.DateTimeFromToRangeFilter()
    is_active = django_filters.BooleanFilter()
    mode = django_filters.CharFilter(lookup_expr='icontains')
    site = django_filters.CharFilter(field_name="user__usersignupsource__site", lookup_expr='iexact')

    def filter_course_id(self, queryset, name, value):
        """
        This custom filter was created to enable filtering by course_id.
        See common.djangoapps.xmodule_django.models
        When doing queries over opaque fields, we need to instance the
        KEY_CLASS of the field with the query string first, and then pass
        this instance to the queryset filter.
        In this case, the KEY_CLASS for course_id field is CourseKey
        """
        if value:
            # CourseKey instance creation will fail if course does not exist
            try:
                # Instantiating CourseKey of the field with query string
                instance = CourseKey.from_string(str(value))
                # Passing instance to queryset filter
                return queryset.filter(course_id=instance)
            except Exception:  # pylint: disable=broad-except
                # If CourseKey instantiation fails, return an empty queryset
                return queryset.none()

        return queryset

    class Meta:
        """
        TODO: add me
        """
        model = get_course_enrollment()
        fields = [
            'id',
            'course_id',
            'created',
            'is_active',
            'mode',
            'site',
        ]


class GeneratedCerticatesFilter(BaseDataApiFilter):
    """
    TODO: add me
    """
    DOWNLOADABLE = 'downloadable'
    ALL = 'all'

    site = django_filters.CharFilter(field_name="user__usersignupsource__site", lookup_expr='iexact')
    username = django_filters.CharFilter(field_name="user__username", lookup_expr='icontains')
    created_date = django_filters.DateTimeFromToRangeFilter()
    course_id = django_filters.CharFilter(method="filter_course_id")
    status = django_filters.CharFilter(method="filter_status")
    grade = django_filters.CharFilter()
    mode = django_filters.CharFilter()

    def filter_course_id(self, queryset, name, value):
        """
        This custom filter was created to enable filtering by course_id.
        See common.djangoapps.xmodule_django.models
        When doing queries over opaque fields, we need to instance the
        KEY_CLASS of the field with the query string first, and then pass
        this instance to the queryset filter.
        In this case, the KEY_CLASS for course_id field is CourseKey
        """
        if value:
            # CourseKey instance creation will fail if course does not exist
            try:
                # Instantiating CourseKey of the field with query string
                instance = CourseKey.from_string(str(value))
                # Passing instance to queryset filter
                return queryset.filter(course_id=instance)
            except Exception:  # pylint: disable=broad-except
                # If CourseKey instantiation fails, return an empty queryset
                return queryset.none()

        return queryset

    def filter_status(self, queryset, name, value):
        """
        This custom filter was created to return a queryset
        where certificates have downloadable status or
        another queryset with all available certificates.
        """
        if value:
            if value == self.DOWNLOADABLE:
                return queryset.filter(status=value)
            if value == self.ALL:
                return queryset.exclude(status=self.DOWNLOADABLE)

        return queryset

    class Meta:
        """
        TODO: add me
        """
        model = get_generated_certificate()
        fields = [
            'id',
            'course_id',
            'grade',
            'mode',
            'username',
            'status',
            'site',
            'created_date'
        ]


class ProctoredExamStudentAttemptFilter(BaseDataApiFilter):
    """
    TODO: add me
    """
    site = django_filters.CharFilter(field_name="user__usersignupsource__site", lookup_expr='iexact')
    course_id = django_filters.CharFilter(field_name="proctored_exam__course_id", lookup_expr='iexact')
    exam_name = django_filters.CharFilter(field_name="proctored_exam__exam_name", lookup_expr='iexact')

    class Meta:
        """
        TODO: add me
        """
        model = ProctoredExamStudentAttempt
        fields = [
            'id',
            'site',
            'course_id',
            'exam_name'

        ]



"""
Extensions to the regular defined third party auth backends
"""
import logging

from django.conf import settings
from django.contrib.auth import get_user_model
from social_core.backends.open_id_connect import OpenIdConnectAuth
from social_core.exceptions import AuthMissingParameter

from eox_core.edxapp_wrapper.configuration_helpers import get_configuration_helper

configuration_helper = get_configuration_helper()  # pylint: disable=invalid-name

LOG = logging.getLogger(__name__)
User = get_user_model()  # pylint: disable=invalid-name


class ConfigurableOpenIdConnectAuth(OpenIdConnectAuth):     # pylint: disable=abstract-method
    """
    Generic backend that can be configured via the site settings
    """
    name = 'config-based-openidconnect'

    def __init__(self, *args, **kwargs):
        conf = configuration_helper.get_value(
            "OIDC_CONFIG",
            getattr(settings, 'OIDC_CONFIG', {})
        )

        for key in conf:
            try:
                setattr(self, key, conf.get(key, getattr(self, key)))
            except Exception:  # pylint: disable=broad-except
                LOG.error("Tried and failed to set property %s of a config-based-openidconnect", key)

        super().__init__(*args, **kwargs)

    def oidc_config(self):
        """
        Override method that gets OICD configuration per class instance.
        """
        LOG.debug(
            "Be aware that ConfigurableOpenIdConnectAuth is using an override "
            "method to get OICD configuration."
        )
        if not hasattr(self, 'configuration'):
            self.configuration = self.get_json(  # pylint: disable=attribute-defined-outside-init
                self.OIDC_ENDPOINT + '/.well-known/openid-configuration'
            )
        return self.configuration

    def get_user_details(self, response):
        """
        Override used to truncate first name in case is longer than the field length.
        """
        user_details = super().get_user_details(response)
        first_name_max_len = User._meta.get_field("first_name").max_length  # pylint: disable=no-member, protected-access
        first_name = user_details["first_name"]
        user_details["first_name"] = first_name[:first_name_max_len] if first_name else first_name

        return user_details

    def extra_data(self, user, uid, response, details=None, *args, **kwargs):  # pylint: disable=keyword-arg-before-vararg
        """
        Get information returned by the provider including user information.

        When debug mode is enabled, the entire response from the provided is
        added. This behavior is controlled via site settings:

        - 'SOCIAL_AUTH_DEBUG_ENABLED': if true, then debug mode is enabled.
        default is set to false.

        Examples:
            - Default mode
            {
                'refresh_token': None,
                ...
                'user_details':
                {
                    'first_name': 'Test',
                    'last_name': 'Subject',
                    ...
                },
                ...
            }
            - Debug mode enabled -after enabled once- or exception raised
            {
                ...
                'user_details': ...,
                'userinfo_response':
                {
                    'locale': 'es',
                    'hd': 'edunext.co',
                    'given_name': 'Test',
                    'email': 'test.subject@edunext.co',
                    'family_name': 'Subject',
                    ...
                    'email_verified': True,
                    'expires_in': 3598,
                    'picture': ...,
            }
        """
        data = super().extra_data(user, uid, response, details, *args, **kwargs)

        debug_mode_enabled = self.setting('DEBUG_ENABLED', False)

        try:
            data["user_details"] = self.get_user_details(response)
            if debug_mode_enabled:
                data["userinfo_response"] = response
        except Exception:  # pylint: disable=broad-except
            LOG.exception("An exception was raised while loading user_details.")
            data["userinfo_response"] = response

        return data

    def get_user_id(self, *args, **kwargs):
        """
        Insert a slug at the beginning of the user id. Retroactively add the
        slug for social users that don't have one. The slug must be defined
        in the 'OTHER_SETTINGS' field from the provider configuration.
        In case a slug was not defined raise an exception due to misconfigured
        backend.
        This three behaviours can be controlled via the settings:
            - 'SOCIAL_AUTH_NAMESPACED_UIDS': if false behave as originally
            intended. If true, add a slug to the uid.
            - 'SOCIAL_AUTH_ALLOW_SLUGLESS_UID': if false raise and exception when no slug
            is defined. If true, return de original uid when no slug is defined.
            - 'SOCIAL_AUTH_ALLOW_WRITE_SLUG_UID': If true update the uid of existing social
            users if any.

        A first step to start migrating to name spaced uids would look like this:

        on the site settings:
        {
          "SOCIAL_AUTH_NAMESPACED_UIDS": true,
          "SOCIAL_AUTH_ALLOW_SLUGLESS_UID": true,
          "SOCIAL_AUTH_ALLOW_WRITE_SLUG_UID": true
        }

        on the provider configuration:
            OTHER_SETTINGS:
            {
              "slug": "myslug"
            }

        Setting 'SOCIAL_AUTH_NAMESPACED_UIDS' to true and adding the slug to
        'OTHER_SETTINGS' will create new associations with the new format.
        'SOCIAL_AUTH_ALLOW_WRITE_SLUG_UID'=true  will update older entries to the new
        format at login time and 'SOCIAL_AUTH_ALLOW_SLUGLESS_UID'=true would allow
        the previous format for older entries for the time being.

        Once all the old uids have been updated to the new format we can forbid the
        old format altogether and stop trying to updated uids without a slug.

        on the site settings:
        {
          "SOCIAL_AUTH_NAMESPACED_UIDS": true,
          "SOCIAL_AUTH_ALLOW_SLUGLESS_UID": false,
          "SOCIAL_AUTH_ALLOW_WRITE_SLUG_UID": false
        }

        on the provider configuration:
            OTHER_SETTINGS:
            {
              "slug": "myslug"
            }
        """
        uid = super().get_user_id(*args, **kwargs)
        namespaced_uids = self.setting('NAMESPACED_UIDS', False)
        if not namespaced_uids:
            return uid

        strategy = self.strategy
        slug = strategy.setting('slug', backend=self)
        provider = self.name
        allow_slugless_uid = self.setting('ALLOW_SLUGLESS_UID', False)
        allow_write_slug_uid = self.setting('ALLOW_WRITE_SLUG_UID', True)

        if not slug:
            if allow_slugless_uid:
                return uid
            raise AuthMissingParameter(self, 'slug')

        slug_uid = f'{slug}:{uid}'
        if allow_write_slug_uid:
            social = strategy.storage.user.get_social_auth(provider, uid)
            if social:
                social.uid = slug_uid
                social.save()
                LOG.info("Updating uid: %s to %s", uid, slug_uid)

        return slug_uid



"""
API v1 views.
"""

# pylint: disable=too-many-lines
from __future__ import absolute_import, unicode_literals

import logging

import edx_api_doc_tools as apidocs
import six
from django.conf import settings
from django.contrib.sites.shortcuts import get_current_site
from edx_rest_framework_extensions.auth.jwt.authentication import JwtAuthentication
from rest_framework import status
from rest_framework.authentication import SessionAuthentication
from rest_framework.exceptions import APIException, NotFound, ValidationError
from rest_framework.renderers import BrowsableAPIRenderer, JSONRenderer
from rest_framework.response import Response
from rest_framework.views import APIView

from eox_core.api.v1.permissions import EoxCoreAPIPermission
from eox_core.api.v1.serializers import (
    EdxappCourseEnrollmentQuerySerializer,
    EdxappCourseEnrollmentSerializer,
    EdxappCoursePreEnrollmentSerializer,
    EdxappGradeSerializer,
    EdxappUserQuerySerializer,
    EdxappUserReadOnlySerializer,
    EdxappUserSerializer,
    WrittableEdxappUserSerializer,
)
from eox_core.edxapp_wrapper.bearer_authentication import BearerAuthentication
from eox_core.edxapp_wrapper.coursekey import get_valid_course_key
from eox_core.edxapp_wrapper.courseware import get_courseware_courses
from eox_core.edxapp_wrapper.enrollments import create_enrollment, delete_enrollment, get_enrollment, update_enrollment
from eox_core.edxapp_wrapper.grades import get_course_grade_factory
from eox_core.edxapp_wrapper.pre_enrollments import (
    create_pre_enrollment,
    delete_pre_enrollment,
    get_pre_enrollment,
    update_pre_enrollment,
)
from eox_core.edxapp_wrapper.users import create_edxapp_user, get_edxapp_user, get_user_read_only_serializer

try:
    from eox_audit_model.decorators import audit_drf_api
except ImportError:
    def audit_drf_api(*args, **kwargs):
        """Identity decorator"""
        return lambda x: x

LOG = logging.getLogger(__name__)


class UserQueryMixin:
    """
    Provides tools to create user queries
    """

    def __init__(self, *args, **kwargs):
        """
        Defines instance attributes
        """
        super().__init__(*args, **kwargs)
        self.query_params = None
        self.site = None

    def initial(self, request, *args, **kwargs):
        """
        Loads the site into the object for every kind of request.
        """
        super().initial(request, *args, **kwargs)

        if hasattr(request, "site"):
            self.site = request.site

    def get_query_params(self, request):
        """
        Utility to read the query params in a forgiving way
        As a side effect it loads self.query_params also in a forgiving way
        """
        query_params = request.query_params
        if not query_params and request.data:
            query_params = request.data

        self.query_params = query_params

        return query_params

    def get_user_query(self, request, query_params=None):
        """
        Utility to prepare the user query
        """
        if not query_params:
            query_params = self.get_query_params(request)

        username = query_params.get("username", None)
        email = query_params.get("email", None)

        if not email and not username:
            raise ValidationError(detail="Email or username needed")

        user_query = {}
        if hasattr(self, "site") and self.site:
            user_query["site"] = self.site
        if username:
            user_query["username"] = username
        elif email:
            user_query["email"] = email

        return user_query


class EdxappUser(UserQueryMixin, APIView):
    """
    Handles the creation of a User on edxapp

    **Example Requests**

        POST /eox-core/api/v1/user/

        Request data: {
            "username": "johndoe",
            "email": "johndoe@example.com",
            "fullname": "John Doe",
            "password": "p@ssword",
        }

    The extra registration fields configured for the microsite, should be sent along with the rest of the parameters.
    These extra fields would be required depending on the settings.
    For example, if we have the microsite settings:

        "EDNX_CUSTOM_REGISTRATION_FIELDS": [
            {
                "label": "Personal ID",
                "name": "personal_id",
                "type": "text"
            },
        ],

        "REGISTRATION_EXTRA_FIELDS": {
            "gender": "required",
            "country": "hidden",
            "personal_id": "required",
        },

        "extended_profile_fields": [
            "personal_id",
        ],

    Then a request to create a user should look like this:

        {
            "username": "johndoe",
            "email": "johndoe@example.com",
            "fullname": "John Doe",
            "password": "p@ssword",
            "gender": "m",
            "country": "GR",
            "personal_id": "12345",
        }

    """

    authentication_classes = (BearerAuthentication, SessionAuthentication, JwtAuthentication)
    permission_classes = (EoxCoreAPIPermission,)
    renderer_classes = (JSONRenderer, BrowsableAPIRenderer)

    @apidocs.schema(
        body=EdxappUserQuerySerializer,
        responses={
            200: EdxappUserSerializer,
            400: "Bad request, a required field is missing or has been entered with the wrong format.",
            401: "Unauthorized user to make the request.",
        },
    )
    @audit_drf_api(
        action="Create edxapp user",
        data_filter=[
            "email",
            "username",
            "fullname",
        ],
        hidden_fields=["password"],
        save_all_parameters=True,
        method_name='eox_core_api_method',
    )
    def post(self, request, *args, **kwargs):
        """
        Handles the creation of a User on edxapp

        **Example Requests**

            POST /eox-core/api/v1/user/

            Request data: {
                "username": "johndoe",
                "email": "johndoe@example.com",
                "fullname": "John Doe",
                "password": "p@ssword",
            }

        **Parameters**

        - `username` (**required**, string, _body_):
            The username to be assigned to the new user.

        - `email` (**required**, string, _body_):
            The email to be assigned to the new user.

        - `password` (**required**, string, _body_):
            The password of the new user. If `skip_password` is True, this field will be omitted.

        - `fullname` (**required**, string, _body_):
            The full name to be assigned.

        - `activate_user` (**optional**, boolean, default=False, _body_):
            Flag indicating whether the user is active.

        - `skip_password` (**optional**, boolean, default=False, _body_):
            Flag indicating whether the password should be omitted.

        If you have extra registration fields configured in your settings or extended_profile fields, you can send them with the rest of the parameters.
        These extra fields would be required depending on the site settings.
        For example:

            {
                "username": "johndoe",
                "email": "johndoe@example.com",
                "fullname": "John Doe",
                "password": "p@ssword",
                "gender": "m",
                "country": "GR",
            }


        **Returns**

        - 200: Success, user created.
        - 400: Bad request, a required field is missing or has been entered with the wrong format, or the chosen email/username already belongs to a user.
        - 401: Unauthorized user to make the request.
        """
        serializer = EdxappUserQuerySerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data
        data["site"] = get_current_site(request)
        user, msg = create_edxapp_user(**data)

        serialized_user = EdxappUserSerializer(user)
        response_data = serialized_user.data
        if msg:
            response_data["messages"] = msg
        return Response(response_data)

    @apidocs.schema(
        parameters=[
            apidocs.query_parameter(
                name="username",
                param_type=str,
                description="**required**, The username used to identify the user. Use either username or email.",
            ),
            apidocs.query_parameter(
                name="email",
                param_type=str,
                description="**required**, The email used to identify the user. Use either username or email.",
            ),
        ],
        responses={
            200: get_user_read_only_serializer(),
            400: "Bad request, missing email or username",
            401: "Unauthorized user to make the request.",
            404: "User not found",
        },
    )
    def get(self, request, *args, **kwargs):
        """
        Retrieves information about an edxapp user,
        given an email or a username.

        The username prevails over the email when both are provided to get the user.

        **Example Requests**

            GET /eox-core/api/v1/user/?username=johndoe

            Query parameters: {
              "username": "johndoe",
            }

        **Response details**

        - `username (str)`: Username of the edxapp user
        - `is_active (str)`: Indicates if the user is active on the platform
        - `email (str)`: Email of the user
        - `gender (str)`: Gender of the user
        - `date_joined (str)`: Date for when the user was registered in the platform
        - `name (str)`: Fullname of the user
        - `country (str)`: Country of the user
        - `level_of_education (str)`: Level of education of the user
        - `year_of_birth (int)`: Year of birth of the user
        - `bio (str)`: Bio of the user
        - `goals (str)`: Goals of the user
        - `extended_profile (list)`: List of dictionaries that contains the user-profile meta fields
            - `field_name (str)`: Name of the extended profile field
            - `field_value (str)`: Value of the extended profile field
        - `mailing_address (str)`
        - `social_links (List)`: List that contains the social links of the user, if any.
        - `account_privacy (str)`: Indicates the account privacy type
        - `state (str)`: State (only for US)
        - `secondary_email (str)`: Secondary email of the user
        - `profile_image (dictionary)`:
            - `has_image (Bool)`: Indicates if user has profile image
            - `image_url_medium (str)`: Url of the profile image in medium size
            - `image_url_small (str)`: Url of the profile image in small size
            - `image_url_full (str)`: Url of the profile image in full size,
            - `image_url_large (str)`: Url of the profile image in large size
        - `secondary_email_enabled (Bool)`: Indicates if the secondary email is enable
        - `phone_number (str)`: Phone number of the user
        - `requires_parental_consent (Bool)`: Indicates whether parental consent is required for the user

        **Returns**

        - 200: Success, user found.
        - 400: Bad request, missing either email or username
        - 401: Unauthorized user to make the request.
        - 404: User not found
        """
        query = self.get_user_query(request)
        user = get_edxapp_user(**query)
        admin_fields = getattr(settings, "ACCOUNT_VISIBILITY_CONFIGURATION", {}).get(
            "admin_fields", {}
        )
        serialized_user = EdxappUserReadOnlySerializer(
            user, custom_fields=admin_fields, context={"request": request}
        )
        response_data = serialized_user.data
        # Show a warning if the request is providing email and username
        # to let the client know we're giving priority to the username
        if set(("email", "username")).issubset(self.query_params):
            response_data[
                "warning"
            ] = "The username prevails over the email when both are provided to get the user."

        return Response(response_data)


class EdxappUserUpdater(UserQueryMixin, APIView):
    """
    Partially updates a user from edxapp.

    Not all the fields can be updated, just the ones thought as `safe`. By default, this fields are the ones defined in the
    EOX_CORE_USER_UPDATE_SAFE_FIELDS

    **Example Requests**

        PATCH /eox-core/api/v1/update-user/

        Request data: {
            "email": "johndoe@example.com",
            "fullname": "John Doe R",
            "password": "new-p@ssword",
        }

    The extra registration fields configured for the microsite, should be sent along with the rest of the parameters.
    These extra fields would be required (their value cannot be changed to null) depending on the settings.

    For example, if we have the microsite settings:

        "EDNX_CUSTOM_REGISTRATION_FIELDS": [
            {
                "label": "Personal ID",
                "name": "personal_id",
                "type": "text"
            },
        ],

        "REGISTRATION_EXTRA_FIELDS": {
            "gender": "required",
            "country": "hidden",
            "personal_id": "required",
        },

        "extended_profile_fields": [
            "personal_id",
        ],

    Then a request to create a user should look like this:

        {
            "email": "johndoe@example.com",
            "fullname": "John Doe R",
            "password": "new-p@ssword",
            "country": "CO",
            "personal_id": "00099",
        }
    """

    authentication_classes = (BearerAuthentication, SessionAuthentication, JwtAuthentication)
    permission_classes = (EoxCoreAPIPermission,)
    renderer_classes = (JSONRenderer, BrowsableAPIRenderer)

    @apidocs.schema(
        body=WrittableEdxappUserSerializer,
        responses={
            200: get_user_read_only_serializer(),
            400: "Bad request, a required field is now null or has been entered with the wrong format.",
            401: "Unauthorized user to make the request.",
            404: "User not found.",
        },
    )
    @audit_drf_api(
        action='Partially update a user from edxapp',
        data_filter=[
            'email',
            'is_active',
        ],
        hidden_fields=['password'],
        save_all_parameters=True,
        method_name='eox_core_api_method',
    )
    def patch(self, request, *args, **kwargs):
        """
        Partially updates a user from edxapp.

        **Example Requests**

            PATCH /eox-core/api/v1/update-user/

            Request data: {
                "email": "johndoe@example.com",
                "fullname": "John Doe R",
                "password": "new-p@ssword",
            }


        **Parameters**

        - `email` (**required**, string, _body_):
            The email used to identify the user. Use either username or email.

        - `username` (**required**, string, _body_):
            The username used to identify the user. Use either username or email.

        - `password` (**optional**, string, _body_):
            The new password of the user.

        - `fullname` (**optional**, string, _body_):
            The full name to be assigned.

        - `is_active` (**optional**, boolean, _body_):
            Flag indicating if the user is active on the platform.

        - Not all the fields can be updated, just the ones thought as 'safe', such as: "is_active", "password", "fullname"

        - By default, these are the 'safe' extra registration fields: "mailing_address", "year_of_birth", "gender", "level_of_education",
        "city", "country", "goals", "bio" and "phone_number".

        If you have extra registration fields configured in your settings or extended_profile fields, and you want to update them, you can send them along with the rest of the parameters.
        For example:

            {
                "email": "johndoe@example.com",
                "fullname": "John Doe R",
                "password": "new-p@ssword",
                "gender": "f",
                "country": "US",
            }

        **Response details**

        - `username (str)`: Username of the edxapp user
        - `is_active (str)`: Indicates if the user is active on the platform
        - `email (str)`: Email of the user
        - `gender (str)`: Gender of the user
        - `date_joined (str)`: Date for when the user was registered in the platform
        - `name (str)`: Fullname of the user
        - `country (str)`: Country of the user
        - `level_of_education (str)`: Level of education of the user
        - `year_of_birth (int)`: Year of birth of the user
        - `bio (str)`: Bio of the user
        - `goals (str)`: Goals of the user
        - `extended_profile (list)`: List of dictionaries that contains the user-profile meta fields
            - `field_name (str)`: Name of the extended profile field
            - `field_value (str)`: Value of the extended profile field
        - `mailing_address (str)`
        - `social_links (List)`: List that contains the social links of the user, if any.
        - `account_privacy (str)`: Indicates the account privacy type
        - `state (str)`: State (only for US)
        - `secondary_email (str)`: Secondary email of the user
        - `profile_image (dictionary)`:
            - `has_image (Bool)`: Indicates if user has profile image
            - `image_url_medium (str)`: Url of the profile image in medium size
            - `image_url_small (str)`: Url of the profile image in small size
            - `image_url_full (str)`: Url of the profile image in full size,
            - `image_url_large (str)`: Url of the profile image in large size
        - `secondary_email_enabled (Bool)`: Indicates if the secondary email is enable
        - `phone_number (str)`: Phone number of the user
        - `requires_parental_consent (Bool)`: Indicates whether parental consent is required for the user

        **Returns**

        - 200: Success, user updated.
        - 400: Bad request, a required field is now null or has been entered with the wrong format.
        - 401: Unauthorized user to make the request.
        - 404: User not found
        """
        # Pop identification
        data = request.data.copy()
        query_params = {
            "email": data.pop("email", None),
            "username": data.pop("username", None),
        }
        query = self.get_user_query(request, query_params=query_params)
        user = get_edxapp_user(**query)

        serializer = WrittableEdxappUserSerializer(user, data=data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()

        admin_fields = getattr(settings, "ACCOUNT_VISIBILITY_CONFIGURATION", {}).get(
            "admin_fields", {}
        )
        serialized_user = EdxappUserReadOnlySerializer(
            user, custom_fields=admin_fields, context={"request": request}
        )
        return Response(serialized_user.data)


class EdxappEnrollment(UserQueryMixin, APIView):
    """
    Handles API requests to create users
    """

    authentication_classes = (BearerAuthentication, SessionAuthentication, JwtAuthentication)
    permission_classes = (EoxCoreAPIPermission,)
    renderer_classes = (JSONRenderer, BrowsableAPIRenderer)

    @apidocs.schema(
        body=EdxappCourseEnrollmentQuerySerializer,
        responses={
            200: EdxappCourseEnrollmentQuerySerializer,
            202: "User doesn't belong to site.",
            400: "Bad request, invalid course_id or missing either email or username.",
        },
    )
    @audit_drf_api(
        action='Create single or bulk enrollments',
        data_filter=[
            'email',
            'username',
            'course_id',
            'mode',
            'is_active',
            'enrollment_attributes',
        ],
        method_name='eox_core_api_method',
    )
    def post(self, request, *args, **kwargs):
        """
        Handle creation of single or bulk enrollments

        **Example Requests**

            POST /eox-core/api/v1/enrollment/

            Request data: {
              "username": "johndoe",
              "course_id": "course-v1:edX+DemoX+Demo_Course",
              "mode": "audit",
              "force": "False",
              "is_active": "False",
              "enrollment_attributes": [
                {
                  "namespace": "credit",
                  "name": "provider_id",
                  "value": "institution_name"
                }
              ]
            }

        **Parameters**

        - `username` (**required**, string, _body_):
            The username used to identify the user you want to enroll.  Use either username or email.

        - `email` (**required**, string, _body_):
            The email used to identify the user you to enroll.  Use either username or email.

        - `course_id` (**required**, string, _body_):
            The id of the course in which you want to enroll the user.

        - `mode` (**required**, string, _body_):
            The course mode for the enrollment.  Must be available for the course.

        - `is_active` (boolean, _body_):
            Flag indicating whether the enrollment is active.

        - `force` (boolean, _body_):
            Flag indicating whether the platform business rules for enrollment must be skipped. When it is true, the enrollment
            is created without looking at the enrollment dates, if the course is full, if the enrollment mode is part of the modes
            allowed by that course and other course settings.

        - `enrollment_attributes` (list, _body_):
            List of enrollment attributes. An enrollment attribute can be used to add extra parameters for a specific course mode.
            It must be a dictionary containing the following:
            - namespace: namespace of the attribute
            - name: name of the attribute
            - value: value of the attribute

        In case the case of bulk enrollments, you must provide a list of dictionaries containing
        the parameters specified above; the same restrictions apply.
        For example:

            [{
              "username": "johndoe",
              "course_id": "course-v1:edX+DemoX+Demo_Course",
              "mode": "audit",
              "is_active": "False",
              "force": "False",
              "enrollment_attributes": [
                {
                  "namespace": "credit",
                  "name": "provider_id",
                  "value": "institution_name"
                }
              ]
             },
             {
              "email": "janedoe@example.com",
              "course_id": "course-v1:edX+DemoX+Demo_Course",
              "mode": "audit",
              "is_active": "True",
              "force": "False",
              "enrollment_attributes": []
             },
            ]

        **Returns**

        - 200: Success, enrollment created.
        - 202: User doesn't belong to site.
        - 400: Bad request, invalid course_id or missing either email or username.
        """
        data = request.data
        return EdxappEnrollment.prepare_multiresponse(
            data, self.single_enrollment_create
        )

    @apidocs.schema(
        body=EdxappCourseEnrollmentQuerySerializer,
        responses={
            200: EdxappCourseEnrollmentQuerySerializer,
            202: "User or enrollment doesn't belong to site.",
            400: "Bad request, invalid course_id or missing either email or username.",
        },
    )
    @audit_drf_api(
        action='Update enrollments on edxapp',
        data_filter=[
            'email',
            'username',
            'course_id',
            'mode',
            'is_active',
            'enrollment_attributes',
        ],
        method_name='eox_core_api_method',
    )
    def put(self, request, *args, **kwargs):
        """
        Update enrollments on edxapp

        **Example Requests**

            PUT /eox-core/api/v1/enrollment/

            Request data: {
              "username": "johndoe",
              "course_id": "course-v1:edX+DemoX+Demo_Course",
              "mode": "audit",
              "is_active": "False",
              "enrollment_attributes": [
                {
                  "namespace": "credit",
                  "name": "provider_id",
                  "value": "institution_name"
                }
              ]
            }

        **Parameters**

        - `username` (**required**, string, _body_):
            The username used to identify a user enrolled on the course. Use either username or email.

        - `email` (**required**, string, _body_):
            The email used to identify a user enrolled on the course. Use either username or email.

        - `course_id` (**required**, string, _body_):
            The course id for the enrollment you want to update.

        - `mode` (**required**, string, _body_):
            The course mode for the enrollment. Must be available for the course.

        - `is_active` (boolean, _body_):
            Flag indicating whether the enrollment is active.

        - `enrollment_attributes` (list, _body_):
            An enrollment attribute can be used to add extra parameters for a specific course mode.
            It must be a dictionary containing the following:
            - namespace: namespace of the attribute
            - name: name of the attribute
            - value: value of the attribute

        **Returns**

        - 200: Success, enrollment updated.
        - 202: User or enrollment doesn't belong to site.
        - 400: Bad request, invalid course_id or missing either email or username.
        """
        data = request.data
        return EdxappEnrollment.prepare_multiresponse(
            data, self.single_enrollment_update
        )

    @apidocs.schema(
        parameters=[
            apidocs.query_parameter(
                name="username",
                param_type=str,
                description="**required**, The username used to identify a user enrolled on the course. Use either username or email.",
            ),
            apidocs.query_parameter(
                name="email",
                param_type=str,
                description="**required**, The email used to identify a user enrolled on the course. Use either username or email.",
            ),
            apidocs.query_parameter(
                name="course_id",
                param_type=str,
                description="**required**, The course id for the enrollment you want to check.",
            ),
        ],
        responses={
            200: EdxappCourseEnrollmentSerializer,
            400: "Bad request, missing course_id or either email or username",
            404: "User or course not found",
        },
    )
    def get(self, request, *args, **kwargs):
        """
        Retrieves enrollment information given a user and a course_id

        **Example Requests**

            GET /eox-core/api/v1/enrollment/?username=johndoe&
            course_id=course-v1:edX+DemoX+Demo_Course

            Request data: {
              "username": "johndoe",
              "course_id": "course-v1:edX+DemoX+Demo_Course",
            }

        **Returns**

        - 200: Success, enrollment found.
        - 400: Bad request, missing course_id or either email or username
        - 404: User or course not found
        """
        user_query = self.get_user_query(request)
        user = get_edxapp_user(**user_query)

        course_id = self.query_params.get("course_id", None)

        if not course_id:
            raise ValidationError(detail="You have to provide a course_id")

        enrollment_query = {
            "username": user.username,
            "course_id": course_id,
        }
        enrollment, errors = get_enrollment(**enrollment_query)

        if errors:
            raise NotFound(detail=errors)
        response = EdxappCourseEnrollmentSerializer(enrollment).data
        return Response(response)

    @apidocs.schema(
        parameters=[
            apidocs.query_parameter(
                name="username",
                param_type=str,
                description="**required**, The username used to identify a user enrolled on the course. Use either username or email.",
            ),
            apidocs.query_parameter(
                name="email",
                param_type=str,
                description="**required**, The email used to identify a user enrolled on the course. Use either username or email.",
            ),
            apidocs.query_parameter(
                name="course_id",
                param_type=str,
                description="**required**, The course id for the enrollment you want to check.",
            ),
        ],
        responses={
            204: "Empty response",
            400: "Bad request, missing course_id or either email or username",
            404: "User or course not found",
        },
    )
    @audit_drf_api(action='Delete enrollment on edxapp.', method_name='eox_core_api_method')
    def delete(self, request, *args, **kwargs):
        """
        Delete enrollment on edxapp

        **Example Requests**

            DELETE /eox-core/api/v1/enrollment/

            Request data: {
              "username": "johndoe",
              "course_id": "course-v1:edX+DemoX+Demo_Course",
            }
        """
        user_query = self.get_user_query(request)
        user = get_edxapp_user(**user_query)

        course_id = self.query_params.get("course_id", None)

        if not course_id:
            raise ValidationError(detail="You have to provide a course_id")

        delete_enrollment(user=user, course_id=course_id)
        return Response(status=status.HTTP_204_NO_CONTENT)

    def single_enrollment_create(self, *args, **kwargs):
        """
        Handle one create at the time
        """
        user_query = self.get_user_query(None, query_params=kwargs)
        user = get_edxapp_user(**user_query)

        enrollments, msgs = create_enrollment(user, **kwargs)
        # This logic block is needed to convert a single bundle_id enrollment in a list
        # of course_id enrollments which are appended to the response individually
        if not isinstance(enrollments, list):
            enrollments = [enrollments]
            msgs = [msgs]
        response_data = []
        for enrollment, msg in zip(enrollments, msgs):
            data = EdxappCourseEnrollmentSerializer(enrollment).data
            if msg:
                data["messages"] = msg
            response_data.append(data)

        return response_data

    def single_enrollment_update(self, *args, **kwargs):
        """
        Handle one update at the time
        """
        user_query = self.get_user_query(None, query_params=kwargs)
        user = get_edxapp_user(**user_query)

        course_id = kwargs.pop("course_id", None)
        if not course_id:
            raise ValidationError(detail="You have to provide a course_id for updates")
        mode = kwargs.pop("mode", None)

        return update_enrollment(user, course_id, mode, **kwargs)

    @staticmethod
    def prepare_multiresponse(request_data, action_method):
        """
        Prepare a multiple part response according to the request_data and the action_method provided
        Args:
            request_data: Data dictionary containing the query or queries to be processed
            action_method: Function to be applied to the queries (create, update)

        Returns: List of responses
        """
        multiple_responses = []
        errors_in_bulk_response = False
        many = isinstance(request_data, list)
        serializer = EdxappCourseEnrollmentQuerySerializer(data=request_data, many=many)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data
        if not isinstance(data, list):
            data = [data]

        for enrollment_query in data:

            try:
                result = action_method(**enrollment_query)
                # The result can be a list if the enrollment was in a bundle
                if isinstance(result, list):
                    multiple_responses += result
                else:
                    multiple_responses.append(result)
            except APIException as error:
                errors_in_bulk_response = True
                enrollment_query["error"] = {
                    "detail": error.detail,
                }
                multiple_responses.append(enrollment_query)

        if many or "bundle_id" in request_data:
            response = multiple_responses
        else:
            response = multiple_responses[0]

        response_status = status.HTTP_200_OK
        if errors_in_bulk_response:
            response_status = status.HTTP_202_ACCEPTED
        return Response(response, status=response_status)

    def handle_exception(self, exc):
        """
        Handle exception: log it
        """
        if isinstance(exc, APIException):
            LOG.error("API Error: %s", repr(exc.detail))

        return super().handle_exception(exc)


class EdxappPreEnrollment(APIView):
    """
    Handles API requests to manage whitelistings (pre-enrollments)
    """

    authentication_classes = (BearerAuthentication, SessionAuthentication, JwtAuthentication)
    permission_classes = (EoxCoreAPIPermission,)
    renderer_classes = (JSONRenderer, BrowsableAPIRenderer)

    @audit_drf_api(action='Create pre-enrollments on edxapp.', method_name='eox_core_api_method')
    def post(self, request, *args, **kwargs):
        """
        Create whitelistings on edxapp
        """
        serializer = EdxappCoursePreEnrollmentSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data
        course_id = data.pop("course_id", None)
        pre_enrollment, warning = create_pre_enrollment(course_id=course_id, **data)
        response_data = EdxappCoursePreEnrollmentSerializer(
            pre_enrollment, context=warning
        ).data
        return Response(response_data)

    @audit_drf_api(action='Update pre-enrollments on edxapp.', method_name='eox_core_api_method')
    def put(self, request, *args, **kwargs):
        """
        Update whitelistings on edxapp
        """
        serializer = EdxappCoursePreEnrollmentSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data
        email = data.get("email")
        course_id = data.get("course_id")
        auto_enroll = data.pop("auto_enroll", False)

        pre_enrollment_query = {
            "email": email,
            "course_id": course_id,
        }

        pre_enrollment = get_pre_enrollment(**pre_enrollment_query)
        update_query = {
            "pre_enrollment": pre_enrollment,
            "auto_enroll": auto_enroll,
        }
        updated_pre_enrollment = update_pre_enrollment(**update_query)
        response = EdxappCoursePreEnrollmentSerializer(updated_pre_enrollment).data
        return Response(response)

    @audit_drf_api(action='Delete pre-enrollments on edxapp.', method_name='eox_core_api_method')
    def delete(self, request, *args, **kwargs):
        """
        Delete whitelistings on edxapp
        """
        query_params = request.query_params
        if not query_params:
            query_params = request.data

        serializer = EdxappCoursePreEnrollmentSerializer(data=query_params)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data
        email = data.get("email")
        course_id = data.get("course_id")

        pre_enrollment_query = {
            "email": email,
            "course_id": course_id,
        }

        pre_enrollment = get_pre_enrollment(**pre_enrollment_query)
        delete_query = {
            "pre_enrollment": pre_enrollment,
        }
        delete_pre_enrollment(**delete_query)
        return Response(status=status.HTTP_204_NO_CONTENT)

    def get(self, request, *args, **kwargs):
        """
        Get whitelistings on edxapp
        """
        query_params = request.query_params
        if not query_params:
            query_params = request.data

        serializer = EdxappCoursePreEnrollmentSerializer(data=query_params)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data
        pre_enrollment = get_pre_enrollment(**data)
        response = EdxappCoursePreEnrollmentSerializer(pre_enrollment).data
        return Response(response)

    def handle_exception(self, exc):
        """
        Handle exception: log it
        """
        data = self.request.data or self.request.query_params
        self.log("API Error", data, exc)
        return super().handle_exception(exc)

    def log(self, desc, data, exception=None):
        """
        log util for this view
        """
        log_data = []
        log_data.append(desc)
        log_data.append("Exception:")
        if isinstance(exception, APIException):
            log_data.append(repr(exception.detail))
        else:
            log_data.append(repr(exception))

        log_data.append("Request data:")
        if not data:
            log_data.append("Empty request")
        else:
            for key, value in data.items():
                log_data.append(f"{key}: {value}")

        LOG.error(" ".join(log_data))


class EdxappGrade(UserQueryMixin, APIView):
    """
    Handles API requests to manage course grades
    """

    authentication_classes = (BearerAuthentication, SessionAuthentication, JwtAuthentication)
    permission_classes = (EoxCoreAPIPermission,)
    renderer_classes = (JSONRenderer, BrowsableAPIRenderer)

    @apidocs.schema(
        parameters=[
            apidocs.query_parameter(
                name="username",
                param_type=str,
                description="**required**, The username used to identify a user enrolled on the course. Use either username or email.",
            ),
            apidocs.query_parameter(
                name="email",
                param_type=str,
                description="**required**, The email used to identify a user enrolled on the course. Use either username or email.",
            ),
            apidocs.query_parameter(
                name="course_id",
                param_type=str,
                description="**required**, The course id for the enrollment you want to check.",
            ),
            apidocs.query_parameter(
                name="detailed",
                param_type=bool,
                description="**optional**, If true include detailed data for each graded subsection",
            ),
            apidocs.query_parameter(
                name="grading_policy",
                param_type=bool,
                description="**optional**, If true include course grading policy.",
            ),
        ],
        responses={
            200: EdxappGradeSerializer,
            400: "Bad request, missing course_id or either email or username",
            404: "User, course or enrollment not found",
        },
    )
    def get(self, request):
        """
        Retrieves Grades information for given a user and course_id

        **Example Requests**

            GET /eox-core/api/v1/grade/?username=johndoe&course_id=course-v1:edX+DemoX+Demo_Course

            Request data: {
              "username": "johndoe",
              "course_id": "course-v1:edX+DemoX+Demo_Course",
            }

        **Response details**

        - `earned_grade`: Final user score for the course.
        - `section_breakdown` (**optional**): Details of each grade subsection.
            - `attempted`: Whether the learner attempted the assignment.
            - `assignment_type`: General category of the assignment.
            - `percent`: Grade obtained by the user on this subsection.
            - `score_earned`: The score a user earned on this subsection.
            - `score_possible`: Highest possible score a user can earn on this subsection.
            - `subsection_name`: Name of the subsection.
        - `grading_policy` (**optional**): Course grading policy.
            - `grade_cutoff`: Score needed to reach each grade.
            - `grader`: Details of each assignment type used by the Grader.
                - `assignment_type`: General category of the assignment.
                - `count`: Number of assignments of this type.
                - `dropped`: The number of assignments of this type that the grader will drop. The grader will drop the lowest-scored assignments first.
                - `weight`: Weight of this type of assignment towards the final grade.

        More information about grading can be found in the
        [edX documentation](https://edx.readthedocs.io/projects/open-edx-building-and-running-a-course/en/latest/student_progress/course_grades.html).

        **Returns**

        - 200: Success.
        - 400: Bad request, missing course_id or either email or username.
        - 404: User, course or enrollment not found.
        """
        user_query = self.get_user_query(request)
        user = get_edxapp_user(**user_query)

        course_id = self.query_params.get("course_id", None)
        detailed = self.query_params.get("detailed", False)
        grading_policy = self.query_params.get("grading_policy", False)

        if not course_id:
            raise ValidationError(detail="You have to provide a course_id")

        _, errors = get_enrollment(username=user.username, course_id=course_id)

        if errors:
            raise NotFound(errors)

        grade_factory = get_course_grade_factory()
        course_key = get_valid_course_key(course_id)
        course = get_courseware_courses().get_course_by_id(course_key)
        course_grade = grade_factory().read(user, course)
        response = {"earned_grade": course_grade.percent}

        if detailed in ("True", "true", "on", "1"):
            breakdown = self._section_breakdown(course_grade.subsection_grades)
            response["section_breakdown"] = breakdown
        if grading_policy in ("True", "true", "on", "1"):
            response["grading_policy"] = course.grading_policy

        return Response(EdxappGradeSerializer(response).data)

    def _section_breakdown(self, subsection_grades):
        """
        Given a list of subsections grades determine if the subsection is
        graded and creates a list of grade data for each subsection.

        This logic comes from `edunext-platform/lms/djangoapps/grades/rest_api/v1/gradebook_views`
        with a few changes in the return value: `label` and `module_id` were
        removed and `category` was renamed to `assignment_type` to keep
        consistency with the names used in `grading_policy`.
        """
        breakdown = []
        for subsection in subsection_grades.values():
            if not subsection.graded:
                continue

            attempted = False
            score_earned = 0
            score_possible = 0

            if subsection.attempted_graded or subsection.override:
                attempted = True
                score_earned = subsection.graded_total.earned
                score_possible = subsection.graded_total.possible

            breakdown.append(
                {
                    "attempted": attempted,
                    "assignment_type": subsection.format,
                    "percent": subsection.percent_graded,
                    "score_earned": score_earned,
                    "score_possible": score_possible,
                    "subsection_name": subsection.display_name,
                }
            )
        return breakdown


class UserInfo(APIView):
    """
    Auth-only view to check some basic info about the current user
    Can use Oauth2/Session
    """

    authentication_classes = (BearerAuthentication, SessionAuthentication, JwtAuthentication)
    permission_classes = (EoxCoreAPIPermission,)
    renderer_classes = (JSONRenderer, BrowsableAPIRenderer)

    def get(self, request, format=None):  # pylint: disable=redefined-builtin
        """
        handle GET request
        """
        content = {
            # `django.contrib.auth.User` instance.
            "user": six.text_type(request.user.username),
            "email": six.text_type(request.user.email),
            "auth": six.text_type(request.auth),
        }

        if request.user.is_staff:
            content["is_superuser"] = request.user.is_superuser
            content["is_staff"] = request.user.is_staff

        return Response(content)



"""
Certificates definitions.
"""

from importlib import import_module

from django.conf import settings


def get_generated_certificate():
    """ Gets GeneratedCertificate model. """

    backend_function = settings.EOX_CORE_CERTIFICATES_BACKEND
    backend = import_module(backend_function)

    return backend.get_generated_certificate()


def generate_certificate_task(**kwargs):
    """Get generate_certificate_task task."""
    backend_function = settings.EOX_CORE_CERTIFICATES_BACKEND
    backend = import_module(backend_function)

    return backend.get_generate_certificate_task(**kwargs)


def get_certificate_url(**kwargs):
    """Get certificate URL function."""
    backend_function = settings.EOX_CORE_CERTIFICATES_BACKEND
    backend = import_module(backend_function)

    return backend.get_certificate_url(**kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Test module for CourseKey under the open-release/hawthorn.beta1 tag
"""
from __future__ import absolute_import, unicode_literals

import mock
from django.conf import settings
from django.test import TestCase

from ..coursekey import get_valid_course_key, validate_org


class CourseKeyTest(TestCase):
    """ Tests correct load of CourseKey module """

    def setUp(self):
        """ setup """
        super().setUp()
        self.m_course_id = "course-v1:org+course+run"

    @mock.patch('eox_core.edxapp_wrapper.coursekey.import_module')
    def test_import_the_backend(self, m_import):
        """ Test we import the correct backend defined in the settings """

        validate_org(self.m_course_id)
        m_import.assert_called_with(settings.EOX_CORE_COURSEKEY_BACKEND)

    @mock.patch('eox_core.edxapp_wrapper.coursekey.import_module')
    def test_call_the_backend(self, m_import):
        """ Test we use the imported backend """
        m_coursekey_backend = mock.MagicMock()
        m_import.return_value = m_coursekey_backend

        validate_org(self.m_course_id)
        m_coursekey_backend.validate_org.assert_called_with(self.m_course_id)

        get_valid_course_key(self.m_course_id)
        m_coursekey_backend.get_valid_course_key.assert_called_with(self.m_course_id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Test module for PreEnrollments under the open-release/hawthorn.beta1 tag
"""
from django.contrib.auth.models import User
from django.test import TestCase
from mock import patch
from rest_framework.test import APIClient


class PreEnrollmentsAPITest(TestCase):
    """ Tests for the pre-enrollments endpoint """

    patch_permissions = patch('eox_core.api.v1.permissions.EoxCoreAPIPermission.has_permission', return_value=True)

    def setUp(self):
        """ setup """
        super().setUp()
        self.api_user = User('test', 'test@example.com', 'test')
        self.client = APIClient()
        self.url = '/api/v1/pre-enrollment/'
        self.client.force_authenticate(user=self.api_user)

    @patch_permissions
    def test_api_get_input_validation(self, _):
        """ Test the parameter validation of GET method """
        # Test empty request
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 400)
        self.assertIn('email', response.data)

        # Test request with only email
        params = {
            'email': 'test@example.com',
        }
        response = self.client.get(self.url, params)
        self.assertEqual(response.status_code, 400)
        self.assertIn('course_id', response.data)

    @patch_permissions
    @patch('eox_core.api.v1.serializers.validate_org')
    @patch('eox_core.api.v1.serializers.get_valid_course_key')
    @patch('eox_core.api.v1.views.get_pre_enrollment')
    def test_api_get(self, m_get_pre_enrollment, m_get_valid_course_key, *_):
        """ Test that the GET method works under normal conditions """
        m_get_pre_enrollment.return_value = None

        params = {
            'email': 'test@example.com',
            'course_id': 'course-v1:org+course+run',
            'auto_enroll': True,
        }
        m_get_valid_course_key.return_value = params['course_id']
        response = self.client.get(self.url, params)
        self.assertEqual(response.status_code, 200)

        m_get_pre_enrollment.assert_called_once_with(**params)

    @patch_permissions
    def test_api_post_input_validation(self, _):
        """ Test the parameter validation of POST method """
        # Test empty request
        response = self.client.post(self.url)
        self.assertEqual(response.status_code, 400)
        self.assertIn('email', response.data)

        # Test request with only email
        params = {
            'email': 'test@example.com',
        }
        response = self.client.post(self.url, params)
        self.assertEqual(response.status_code, 400)
        self.assertIn('course_id', response.data)

    @patch_permissions
    @patch('eox_core.api.v1.serializers.validate_org')
    @patch('eox_core.api.v1.serializers.get_valid_course_key')
    @patch('eox_core.api.v1.views.create_pre_enrollment')
    def test_api_post(self, m_create_pre_enrollment, m_get_valid_course_key, *_):
        """ Test that the POST method works under normal conditions """
        m_create_pre_enrollment.return_value = None, None

        # Test create pre-enrollment to single course
        params = {
            'email': 'test@example.com',
            'course_id': 'course-v1:org+course+run',
            'auto_enroll': True,
        }
        m_get_valid_course_key.return_value = params['course_id']
        response = self.client.post(self.url, params)
        self.assertEqual(response.status_code, 200)

        m_create_pre_enrollment.assert_called_once_with(**params)

    @patch_permissions
    def test_api_put_input_validation(self, _):
        """ Test the parameter validation of PUT method """
        # Test empty request
        response = self.client.put(self.url)
        self.assertEqual(response.status_code, 400)
        self.assertIn('email', response.data)

        # Test request with only email
        params = {
            'email': 'test@example.com',
        }
        response = self.client.put(self.url, params)
        self.assertEqual(response.status_code, 400)
        self.assertIn('course_id', response.data)

    @patch_permissions
    @patch('eox_core.api.v1.serializers.validate_org')
    @patch('eox_core.api.v1.serializers.get_valid_course_key')
    @patch('eox_core.api.v1.views.get_pre_enrollment')
    @patch('eox_core.api.v1.views.update_pre_enrollment')
    def test_api_put(self, m_update_pre_enrollment, m_get_pre_enrollment, m_get_valid_course_key, *_):
        """ Test that the PUT method works under normal conditions """
        m_update_pre_enrollment.return_value = None
        m_pre_enrollment = {
            "warning": "",
            "course_id": "course-v1:org+course+run",
            "auto_enroll": True,
            "email": "test@example.com"
        }
        m_get_pre_enrollment.return_value = m_pre_enrollment
        m_get_valid_course_key.return_value = m_pre_enrollment['course_id']

        # Test update pre-enrollment
        params = {
            'email': 'test@example.com',
            'course_id': 'course-v1:org+course+run',
            'auto_enroll': True,
        }
        response = self.client.put(self.url, params)
        self.assertEqual(response.status_code, 200)

        m_update_pre_enrollment.assert_called_once_with(
            auto_enroll=True,
            pre_enrollment=m_pre_enrollment,
        )

        m_get_pre_enrollment.assert_called_once_with(
            email='test@example.com',
            course_id='course-v1:org+course+run'
        )

    @patch_permissions
    def test_api_delete_validation(self, _):
        """ Test the parameter validation of DELETE method """
        # Test empty request
        response = self.client.delete(self.url)
        self.assertEqual(response.status_code, 400)
        self.assertIn('email', response.data)

        # Test request with only email
        params = {
            'email': 'test@example.com',
        }
        response = self.client.delete(self.url, params)
        self.assertEqual(response.status_code, 400)
        self.assertIn('course_id', response.data)

    @patch_permissions
    @patch('eox_core.api.v1.serializers.validate_org')
    @patch('eox_core.api.v1.serializers.get_valid_course_key')
    @patch('eox_core.api.v1.views.get_pre_enrollment')
    @patch('eox_core.api.v1.views.delete_pre_enrollment')
    def test_api_delete(self, m_delete_pre_enrollment, m_get_pre_enrollment, m_get_valid_course_key, *_):
        """ Test that the DELETE method works under normal conditions """
        m_delete_pre_enrollment.return_value = None
        m_pre_enrollment = {
            "warning": "",
            "course_id": "course-v1:org+course+run",
            "auto_enroll": True,
            "email": "test@example.com"
        }
        m_get_pre_enrollment.return_value = m_pre_enrollment
        m_get_valid_course_key.return_value = m_pre_enrollment['course_id']
        # Test delete pre-enrollment
        params = {
            'email': 'test@example.com',
            'course_id': 'course-v1:org+course+run',
            'auto_enroll': True,
        }
        response = self.client.delete(self.url, params)
        self.assertEqual(response.status_code, 204)

        m_delete_pre_enrollment.assert_called_once_with(
            pre_enrollment=m_pre_enrollment,
        )

        m_get_pre_enrollment.assert_called_once_with(
            email='test@example.com',
            course_id='course-v1:org+course+run'
        )



"""
View for the tasks dispatcher API
"""
import logging
from importlib import import_module

import celery
from django.conf import settings
from django.contrib.sites.shortcuts import get_current_site
from edx_rest_framework_extensions.auth.jwt.authentication import JwtAuthentication
from rest_framework.permissions import IsAdminUser
from rest_framework.response import Response
from rest_framework.status import HTTP_400_BAD_REQUEST, HTTP_404_NOT_FOUND
from rest_framework.views import APIView

from eox_core.edxapp_wrapper.bearer_authentication import BearerAuthentication

try:
    from eox_audit_model.decorators import audit_drf_api
except ImportError:
    def audit_drf_api(*args, **kwargs):
        """Identity decorator"""
        return lambda x: x

LOG = logging.getLogger(__name__)


class TaskAPI(APIView):
    """
    Celery task dispatcher API
    """

    authentication_classes = (BearerAuthentication, JwtAuthentication)
    permission_classes = [IsAdminUser]

    def get(self, request):
        """
        Check the status of a celery task given an id.

            GET /eox-core/tasks-api/v1/tasks/{id}

        parameters:
            task_id(path): task identifier

        example response:
            {
            "result": {"updated_emails": 23},
            "status": "READY
            }
        """
        params = request.query_params
        task_id = params.get("id")

        if not task_id:
            return Response({"detail": "Task id needed"}, status=HTTP_400_BAD_REQUEST)

        async_result = celery.current_app.AsyncResult(id=task_id)

        result = async_result.result if async_result.ready() else {}

        return Response({"result": str(result), "status": str(async_result.status)})

    @audit_drf_api(action='eox-core Task dispatcher.', method_name='eox_core_api_method')
    def post(self, request):
        """
        Dispatches a task to a celery worker. The task must be registered in the worker
        and have to be enabled in the setting EOX_CORE_ASYNC_TASKS.

        Some async task require context from the original request, if a key 'context'
        is included in the task_parameters the site id will be added as a value.
        Original values will be discarded.

            POST /eox-core/tasks-api/v1/tasks/

        parameters:
            task_path(body): Full path to the task
            task_parameters(body): Dictionary with the keyword arguments of the task
            and their values.

        example body:
            {
                "task_path": "path.to.example.task"
                "task_params" {
                    "email": "johndoe@email.com"
                    "username": "john_doe"
                }
            }

        example response:
            { task_id: "7faf4b00-b526-4787-8ef2-f543dadfdb09" }
        """
        data = request.data
        task_path = data.get("task_path", "")
        task_parameters = data.get("task_parameters", {})
        site_id = get_current_site(request).id

        if "context" in task_parameters:
            task_parameters['context'] = {"site_id": site_id}
        enabled_tasks = settings.EOX_CORE_ASYNC_TASKS

        registered_tasks = celery.current_app.tasks

        if task_path not in enabled_tasks or task_path not in registered_tasks:
            return Response({"detail": "Invalid task path"}, status=HTTP_404_NOT_FOUND)

        task = self.import_task(task_path)

        if task:
            result = task.delay(**task_parameters)
            LOG.info("Task %s with params %s", task_path, task_parameters)
            return Response({"task_id": result.id})

        return Response({"detail": "Module not found"}, status=HTTP_404_NOT_FOUND)

    @staticmethod
    def import_task(task_fullpath):
        """
        Given the full path to the function in dot notation import
        the module and return the function reference
        """
        module_path, task_name = task_fullpath.rsplit(".", 1)
        try:
            module = import_module(module_path)
            return getattr(module, task_name)
        except ImportError:
            LOG.error("ImportError while importing %s", module_path)
            return None
        except AttributeError:
            LOG.error("AttributeError while importing %s", task_name)
            return None



"""
TODO: add me
"""
from __future__ import unicode_literals

import six
from rest_framework import relations


class CustomRelatedField(relations.RelatedField):  # pylint: disable=abstract-method
    """
    A read only field that represents its targets using the site field
    """

    def __init__(self, **kwargs):
        kwargs['read_only'] = True
        self.field = kwargs.pop('field')
        super().__init__(**kwargs)

    def to_representation(self, value):
        return six.text_type(getattr(value, self.field))



"""
Support API v1 serializers.
"""
# pylint: disable=abstract-method
from __future__ import absolute_import, unicode_literals

from django.utils import timezone
from oauth2_provider.models import Application
from rest_framework import serializers

from eox_core.api.v1.serializers import MAX_SIGNUP_SOURCES_ALLOWED
from eox_core.edxapp_wrapper.users import (
    check_edxapp_account_conflicts,
    get_user_signup_source,
    get_username_max_length,
)

UserSignupSource = get_user_signup_source()  # pylint: disable=invalid-name

USERNAME_MAX_LENGTH = get_username_max_length()


class WrittableEdxappRemoveUserSerializer(serializers.Serializer):
    """
    Handles the serialization when a user is being removed.
    """
    case_id = serializers.CharField(write_only=True, default=timezone.now().strftime('%Y%m%d%H%M%S'))
    is_support_user = serializers.BooleanField(default=True)


class WrittableEdxappUsernameSerializer(serializers.Serializer):
    """
    Handles the serialization of the data required to update the username of an edxapp user.
    """
    new_username = serializers.CharField(max_length=USERNAME_MAX_LENGTH, write_only=True)

    def validate(self, attrs):
        """
        When a username update is being made, then it checks that:
            - The new username is not already taken by other user.
            - The user is not staff or superuser.
            - The user has just one signup source.
        """
        username = attrs.get("new_username")
        conflicts = check_edxapp_account_conflicts(None, username)
        if conflicts:
            raise serializers.ValidationError({"detail": "An account already exists with the provided username."})

        if self.instance.is_staff or self.instance.is_superuser:
            raise serializers.ValidationError({"detail": "You can't update users with roles like staff or superuser."})

        if UserSignupSource.objects.filter(user__email=self.instance.email).count() > MAX_SIGNUP_SOURCES_ALLOWED:
            raise serializers.ValidationError({"detail": "You can't update users with more than one sign up source."})

        return attrs

    def update(self, instance, validated_data):
        """
        Method to update username of edxapp User.
        """
        key = 'username'
        if validated_data:
            setattr(instance, key, validated_data['new_username'])
            instance.save()

        return instance


class OauthApplicationUserSerializer(serializers.Serializer):
    """
    Oauth Application owner serializer.
    """
    email = serializers.EmailField()
    username = serializers.CharField(max_length=USERNAME_MAX_LENGTH)
    fullname = serializers.CharField(max_length=255, write_only=True)
    permissions = serializers.ListField(
        child=serializers.CharField(),
        allow_null=True,
        write_only=True,
        required=False,
    )


class OauthApplicationSerializer(serializers.ModelSerializer):
    """
    Oauth Application model serializer.
    """
    user = OauthApplicationUserSerializer()

    class Meta:
        """Meta class."""
        model = Application
        read_only_fields = ('created', 'updated', )
        fields = '__all__'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
User model wrapper for cs_comments_service public definition.
"""

from importlib import import_module

from django.conf import settings


def replace_username_cs_user(*args, **kwargs):
    """ Gets the User model wrapper for comments service"""

    backend_function = settings.EOX_CORE_COMMENTS_SERVICE_USERS_BACKEND
    backend = import_module(backend_function)

    return backend.replace_username_cs_user(*args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Test module for the Support API permissions class
"""
from django.test import TestCase
from mock import MagicMock
from rest_framework.exceptions import NotAuthenticated

from eox_core.api.support.v1.permissions import EoxCoreSupportAPIPermission


class SupportAPIPermissionsTest(TestCase):
    """ Tests for the Support API permissions """

    def test_permissions_for_staff(self):
        """ If the user is staff, then it always has permission. """
        request = MagicMock()
        request.user.is_staff = True

        has_perm = EoxCoreSupportAPIPermission().has_permission(request, MagicMock())

        self.assertTrue(has_perm)

    def test_permissions_fails(self):
        """ Tests that if the user is not staff, the NotAuthenticated exception is raised """
        request = MagicMock()
        request.user.is_staff = False

        with self.assertRaises(NotAuthenticated):
            EoxCoreSupportAPIPermission().has_permission(request, MagicMock())

    def test_permissions_without_auth(self):
        """ Tests that if the auth fails, the NotAuthenticated exception is raised """
        request = MagicMock()
        request.user.is_staff = False
        request.user.has_perm.return_value = False
        request.auth = None

        with self.assertRaises(NotAuthenticated):
            EoxCoreSupportAPIPermission().has_permission(request, MagicMock())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Backend for the pre-enrollment (white listings) functionality that works under the open-release/lilac.master tag
"""
# pylint: disable=import-error, protected-access
import logging

from common.djangoapps.student.models import CourseEnrollmentAllowed  # pylint: disable=no-name-in-module
from django.db import IntegrityError
from rest_framework.exceptions import NotFound

from eox_core.edxapp_wrapper.coursekey import get_valid_course_key
from eox_core.edxapp_wrapper.courseware import get_courseware_courses

LOG = logging.getLogger(__name__)


def create_pre_enrollment(*args, **kwargs):
    """
    Create pre-enrollment of given user in the course provided.

    Example:
        >>>create_pre_enrollment(
            {
            "email": "bob@example.com",
            "course_id": course-v1-edX-DemoX-1T2015",
            "auto_enroll": "False"
            }
        )
    """
    warnings = []
    email = kwargs.get('email')
    auto_enroll = kwargs.get('auto_enroll', False)
    course_id = kwargs.pop('course_id')

    try:
        course_key = get_valid_course_key(course_id)
        pre_enrollment = CourseEnrollmentAllowed.objects.create(course_id=course_key, **kwargs)
        # Check if the course exists otherwise add a warning
        course = get_courseware_courses().get_course(course_key)
        LOG.info('Creating regular pre-enrollment for email: %s course_id: %s auto_enroll: %s', email, course.id, auto_enroll)
    except IntegrityError:
        pre_enrollment = None
        raise NotFound(f'Pre-enrollment already exists for email: {email} course_id: {course_id}') from IntegrityError
    except ValueError:
        warnings = [f'Course with course_id:{course_id} does not exist']
    return pre_enrollment, warnings


def update_pre_enrollment(*args, **kwargs):
    """
    Update pre-enrollment of given user in the course provided.

    Example:
        >>>update_pre_enrollment(
            {
            "email": "bob@example.com",
            "course_id": course-v1-edX-DemoX-1T2015",
            "auto_enroll": "False"
            }
        )
    """
    auto_enroll = kwargs.pop('auto_enroll', False)
    pre_enrollment = kwargs.get('pre_enrollment')
    try:
        pre_enrollment.auto_enroll = auto_enroll
        pre_enrollment.save()
        LOG.info('Updating regular pre-enrollment for email: %s course_id: %s auto_enroll: %s', pre_enrollment.email, pre_enrollment.course_id, auto_enroll)
    except Exception:  # pylint: disable=broad-except
        raise NotFound(f'Pre-enrollment not found for email: {pre_enrollment.email} course_id: {pre_enrollment.course_id}') from Exception
    return pre_enrollment


def delete_pre_enrollment(*args, **kwargs):
    """
    Delete pre-enrollment of given user in the course provided.

    Example:
        >>>delete_pre_enrollment(
            {
            "email": "bob@example.com",
            "course_id": course-v1-edX-DemoX-1T2015"
            }
        )
    """
    pre_enrollment = kwargs.get('pre_enrollment')
    try:
        LOG.info('Deleting regular pre-enrollment for email: %s course_id: %s', pre_enrollment.email, pre_enrollment.course_id)
        pre_enrollment.delete()
    except Exception:  # pylint: disable=broad-except
        raise NotFound(f'Pre-enrollment not found for email: {pre_enrollment.email} course_id: {pre_enrollment.course_id}') from Exception


def get_pre_enrollment(*args, **kwargs):
    """
    Get pre-enrollment of given user in the course provided.

    Example:
        >>>get_pre_enrollment(
            {
            "email": "bob@example.com",
            "course_id": course-v1-edX-DemoX-1T2015"
            }
        )
    """
    email = kwargs.get('email')
    course_id = kwargs.pop('course_id')
    try:
        course_key = get_valid_course_key(course_id)
        pre_enrollment = CourseEnrollmentAllowed.objects.get(course_id=course_key, email=email)
        LOG.info('Getting regular pre-enrollment for email: %s course_id: %s', email, course_id)
    except CourseEnrollmentAllowed.DoesNotExist:
        raise NotFound(f'Pre-enrollment not found for email: {email} course_id: {course_id}') from CourseEnrollmentAllowed.DoesNotExist
    return pre_enrollment



"""
Swagger view generator
"""
from django.conf import settings
from django.urls import re_path
from drf_yasg.generators import OpenAPISchemaGenerator
from drf_yasg.openapi import SwaggerDict
from drf_yasg.views import get_schema_view
from edx_api_doc_tools import get_docs_cache_timeout, make_api_info
from rest_framework import permissions

from eox_core.api.v1 import views


class APISchemaGenerator(OpenAPISchemaGenerator):
    """
    Schema generator for eox-core.

    Define an exclusive base path to perform requests for each endpoint and a
    specific security definition using oauth without overwritting project wide
    settings.
    """

    def get_schema(self, request=None, public=False):
        schema = super().get_schema(request, public)
        schema.base_path = '/eox-core/api/v1/'
        return schema

    def get_security_definitions(self):
        security_definitions = {
            'OAuth2': {
                'flow': 'application',
                'tokenUrl': f'{settings.LMS_ROOT_URL}/oauth2/access_token/',
                'type': 'oauth2',
            },
        }
        security_definitions = SwaggerDict.as_odict(security_definitions)
        return security_definitions


api_urls = [  # pylint: disable=invalid-name
    re_path(r'^enrollment/$', views.EdxappEnrollment.as_view(), name='edxapp-enrollment'),
    re_path(r'^grade/$', views.EdxappGrade.as_view(), name='edxapp-grade'),
    re_path(r'^user/$', views.EdxappUser.as_view(), name='edxapp-user'),
]

if getattr(settings, "EOX_CORE_ENABLE_UPDATE_USERS", None):
    api_urls += [
        re_path(r'^update-user/$', views.EdxappUserUpdater.as_view(), name='edxapp-user-updater'),
    ]

api_info = make_api_info(  # pylint: disable=invalid-name
    title="eox core",
    version="v1",
    email=" contact@edunext.co",
    description="REST APIs to interact with edxapp",
)

docs_ui_view = get_schema_view(  # pylint: disable=invalid-name
    api_info,
    generator_class=APISchemaGenerator,
    public=True,
    permission_classes=(permissions.AllowAny,),
    patterns=api_urls,
).with_ui('swagger', cache_timeout=get_docs_cache_timeout())



"""
Test module for the Grades API
"""
from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse
from mock import MagicMock, patch
from rest_framework import status
from rest_framework.test import APIClient

from eox_core.api.v1.views import EdxappGrade


class TestGradesAPI(TestCase):
    """ Tests for the grades endpoints """

    patch_permissions = patch(
        "eox_core.api.v1.permissions.EoxCoreAPIPermission.has_permission",
        return_value=True,
    )

    def setUp(self):
        """ setup """
        self.api_user = User(1, "test@example.com", "test")
        self.client = APIClient()
        self.client.force_authenticate(user=self.api_user)
        self.url = reverse("eox-api:eox-api:edxapp-grade")

    @patch("eox_core.api.v1.views.get_courseware_courses")
    @patch("eox_core.api.v1.views.get_valid_course_key")
    @patch("eox_core.api.v1.views.get_course_grade_factory")
    @patch("eox_core.api.v1.views.get_enrollment")
    @patch("eox_core.api.v1.views.get_edxapp_user")
    @patch_permissions
    def test_get_grade_no_detail_no_policy(  # pylint: disable=too-many-arguments
            self,
            _,
            get_edxapp_user,
            get_enrollment,
            grade_factory,
            get_valid_course_key,
            get_courseware_courses,
    ):
        """Test that the GET method works with default parameters"""

        get_edxapp_user.return_value.username = "test"
        get_enrollment.return_value = None, None
        grade_factory.return_value.return_value.read.return_value.percent = 0.5
        params = {
            "course_id": "course-v1:org+course+run",
            "username": "test",
        }
        expected_response = {"earned_grade": 0.5}

        response = self.client.get(self.url, params)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, expected_response)

    @patch("eox_core.api.v1.views.get_courseware_courses")
    @patch("eox_core.api.v1.views.get_valid_course_key")
    @patch("eox_core.api.v1.views.get_course_grade_factory")
    @patch("eox_core.api.v1.views.get_enrollment")
    @patch("eox_core.api.v1.views.get_edxapp_user")
    @patch_permissions
    def test_get_grade_detail_no_policy(  # pylint: disable=too-many-arguments
            self,
            _,
            get_edxapp_user,
            get_enrollment,
            grade_factory,
            get_valid_course_key,
            get_courseware_courses,
    ):
        """Test that the GET method works including section details"""

        get_edxapp_user.return_value.username = "test"
        get_enrollment.return_value = None, None
        grade_factory.return_value.return_value.read.return_value.percent = 0.5
        grade_factory.return_value.return_value.read.return_value.subsection_grades = {}
        params = {
            "course_id": "course-v1:org+course+run",
            "username": "test",
            "detailed": True,
        }
        expected_response = {"earned_grade": 0.5, "section_breakdown": []}

        response = self.client.get(self.url, params)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, expected_response)

    @patch("eox_core.api.v1.views.get_courseware_courses")
    @patch("eox_core.api.v1.views.get_valid_course_key")
    @patch("eox_core.api.v1.views.get_course_grade_factory")
    @patch("eox_core.api.v1.views.get_enrollment")
    @patch("eox_core.api.v1.views.get_edxapp_user")
    @patch_permissions
    def test_get_grade_detail_policy(  # pylint: disable=too-many-arguments
            self,
            _,
            get_edxapp_user,
            get_enrollment,
            grade_factory,
            get_valid_course_key,
            get_courseware_courses,
    ):
        """Test that the GET method works with section details and grading policy"""

        get_edxapp_user.return_value.username = "test"
        get_enrollment.return_value = None, None
        grade_factory.return_value.return_value.read.return_value.percent = 0.5
        grade_factory.return_value.return_value.read.return_value.subsection_grades = {}
        get_courseware_courses.return_value.get_course_by_id.return_value.grading_policy = {
            "GRADE_CUTOFFS": {},
            "GRADER": [],
        }
        params = {
            "username": "test",
            "course_id": "course-v1:org+course+run",
            "grading_policy": True,
            "detailed": True,
        }
        expected_response = {
            "earned_grade": 0.5,
            "grading_policy": {"grade_cutoffs": {}, "grader": []},
            "section_breakdown": [],
        }

        response = self.client.get(self.url, params)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, expected_response)

    @patch("eox_core.api.v1.views.get_enrollment")
    @patch("eox_core.api.v1.views.get_edxapp_user")
    @patch_permissions
    def test_get_grade_input_validation(self, _, get_edxapp_user, get_enrollment):
        """Test that the GET method fails if not all required parameters are provided"""
        params1 = {"username": "test"}
        params2 = {"course_id": "test:Course"}

        response1 = self.client.get(self.url)
        response2 = self.client.get(self.url, params1)
        response3 = self.client.get(self.url, params2)

        self.assertEqual(response1.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response2.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response3.status_code, status.HTTP_400_BAD_REQUEST)

    def test_grade_section_breakdown(self):
        # pylint: disable=protected-access
        """Test the `_section_breakdown` helper function"""
        subsection_grade1 = MagicMock()
        subsection_grade2 = MagicMock()

        subsection_grade1.graded = True
        subsection_grade1.attempted_graded = True
        subsection_grade1.graded_total.earned = 10
        subsection_grade1.graded_total.possible = 10
        subsection_grade1.percent_graded = 1
        subsection_grade1.format = "Homework"
        subsection_grade1.display_name = "Homework - Questions"

        subsection_grade2.graded = False

        subsection_grades = {1: subsection_grade1, 2: subsection_grade2}
        section_breakdown = EdxappGrade()._section_breakdown(subsection_grades)

        expected_subgrades = [
            {
                "attempted": True,
                "assignment_type": "Homework",
                "percent": 1,
                "score_earned": 10,
                "score_possible": 10,
                "subsection_name": "Homework - Questions",
            }
        ]

        self.assertEqual(section_breakdown, expected_subgrades)



""" Tests for public user creation API. """
from __future__ import absolute_import, unicode_literals

import mock
from django.conf import settings
from django.test import TestCase

from ..enrollments import create_enrollment


class CreateEdxappUserTest(TestCase):
    """ Tests for the public API module """

    @mock.patch('eox_core.edxapp_wrapper.enrollments.import_module')
    def test_import_the_backend(self, m_import):
        """ Test we import the correct backend defined in the settings """

        create_enrollment()
        m_import.assert_called_with(settings.EOX_CORE_ENROLLMENT_BACKEND)

    @mock.patch('eox_core.edxapp_wrapper.enrollments.import_module')
    def test_call_the_backend(self, m_import):
        """ Test we use the imported backend """
        m_enrollment_backend = mock.MagicMock()
        m_import.return_value = m_enrollment_backend

        data = {
            "email": "something",
            "username": "something",
        }

        create_enrollment(data)
        m_enrollment_backend.create_enrollment.assert_called_with(data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Storages public function definitions
"""
from importlib import import_module

from django.conf import settings


def get_edxapp_production_staticfiles_storage():  # pylint: disable=invalid-name
    """
    Return the edx-platform production staticfiles storage
    """
    backend_function = settings.EOX_CORE_STORAGES_BACKEND
    backend = import_module(backend_function)

    return backend.get_edxapp_production_staticfiles_storage()


def get_edxapp_development_staticfiles_storage():  # pylint: disable=invalid-name
    """
    Return the edx-platform production staticfiles storage
    """
    backend_function = settings.EOX_CORE_STORAGES_BACKEND
    backend = import_module(backend_function)

    return backend.get_edxapp_development_staticfiles_storage()



"""
Setup file for eox_core Django plugin.
"""
#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re

from setuptools import setup


with open("README.rst", "r") as fh:
    README = fh.read()

def get_version(*file_paths):
    """Retrieves the version from the main app __init__.py"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')

VERSION = get_version("eox_core", "__init__.py")


def load_requirements(*requirements_paths):
    """
    Load all requirements from the specified requirements files.
    Returns:
        list: Requirements file relative path strings
    """
    requirements = set()
    for path in requirements_paths:
        with open(path, 'r', encoding='utf-8') as requirements_file:
            requirements.update(
                line.split('#')[0].strip() for line in requirements_file.readlines()
                if is_requirement(line.strip())
            )
    return list(requirements)


def is_requirement(line):
    """
    Return True if the requirement line is a package requirement.
    Returns:
        bool: True if the line is not blank, a comment, a URL, or
              an included file
    """
    return line and not line.startswith(('-r', '#', '-e', 'git+', '-c'))


setup(
    name="eox-core",
    python_requires='>=3.8',
    version=VERSION,
    author="eduNEXT",
    author_email="contact@edunext.co",
    url="https://github.com/eduNEXT/eox-core",
    description="eduNEXT Openedx extensions",
    long_description=README,
    long_description_content_type='text/x-rst',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django :: 3.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.10',
    ],
    install_requires=load_requirements('requirements/base.in'),
    extras_require={
        "sentry": load_requirements('requirements/sentry.in'),
        "tpa": load_requirements('requirements/tpa.in'),
        "eox-audit": load_requirements('requirements/eox-audit-model.in')
    },
    scripts=[],
    license="AGPL",
    platforms=["any"],
    zip_safe=False,
    packages=['eox_core'],
    include_package_data=True,
    entry_points={
        "lms.djangoapp": [
            "eox_core = eox_core.apps:EoxCoreConfig",
        ],
        "cms.djangoapp": [
            "eox_core = eox_core.apps:EoxCoreCMSConfig",
        ],
    }
)



#!/usr/bin/python
"""
This file implements the Middleware support for the Open edX platform.
A microsite enables the following features:
1) Mapping of sub-domain name to a 'brand', e.g. foo-university.edx.org
2) Present a landing page with a listing of courses that are specific to the 'brand'
3) Ability to swap out some branding elements in the website
"""
import logging
import re
from urllib.parse import urlparse

import six
from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.views import redirect_to_login
from django.db import IntegrityError
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.http import Http404, HttpResponseRedirect
from django.urls import reverse
from django.utils.deprecation import MiddlewareMixin
from requests.exceptions import HTTPError
from social_core.exceptions import AuthAlreadyAssociated, AuthFailed, AuthUnreachableProvider

from eox_core.edxapp_wrapper.configuration_helpers import get_configuration_helper
from eox_core.edxapp_wrapper.third_party_auth import get_tpa_exception_middleware
from eox_core.models import Redirection
from eox_core.utils import cache, fasthash

try:
    from eox_tenant.pipeline import EoxTenantAuthException
except ImportError:

    class EoxTenantAuthException:
        """Dummy eox-tenant Exception."""


configuration_helper = get_configuration_helper()  # pylint: disable=invalid-name
ExceptionMiddleware = get_tpa_exception_middleware()

LOG = logging.getLogger(__name__)


class PathRedirectionMiddleware(MiddlewareMixin):
    """
    Middleware to create custom responses based on the request path
    """

    def process_request(self, request):
        """
        Process the path of every request and
        determine the correct custom redirect.

        Each custom_redirection_setting is equal in terms of importance,
        but to have order, 'MKTG_REDIRECTS' is defined
        as the one with highest priority.
        """

        custom_redirection_settings = {
            "MKTG_REDIRECTS": "process_mktg_redirect",
            "EDNX_CUSTOM_PATH_REDIRECTS": "process_custom_path_redirect",
        }

        for key, value in six.iteritems(custom_redirection_settings):
            if configuration_helper.has_override_value(key):
                action = getattr(self, value)
                response = action(request)
                if response:
                    return response

        return None

    def process_custom_path_redirect(self, request):
        """
        Redirect the request according to the configured action to take.
        """
        redirects = configuration_helper.get_value("EDNX_CUSTOM_PATH_REDIRECTS", {})

        path = request.path_info

        for regex, values in six.iteritems(redirects):

            if isinstance(values, dict):
                key = next(iter(values))
            else:
                key = values

            regex_path_match = re.compile(regex.format(
                COURSE_ID_PATTERN=settings.COURSE_ID_PATTERN,
                USERNAME_PATTERN=settings.USERNAME_PATTERN,
            ))

            if regex_path_match.match(path):
                try:
                    action = getattr(self, key)
                    return action(request=request, key=key, values=values, path=path)
                except Http404:  # we expect 404 to be raised
                    raise
                except Exception as error:  # pylint: disable=broad-except
                    LOG.error("The PathRedirectionMiddleware generated an error at: %s%s",
                              request.get_host(),
                              request.get_full_path())
                    LOG.error(error)
                    return None
        return None

    def process_mktg_redirect(self, request):
        """
        Redirect the request if there are mktg custom settings
        present that match the request path.
        """
        redirects = configuration_helper.get_value("MKTG_REDIRECTS", {})
        path = request.path_info

        for key, value in six.iteritems(redirects):

            # Strip off html extension to have backwards
            # compatibility to keys defined with template style.
            key = key.replace('.html', '')
            key = f'/{key}'

            # Just continue if the path does not match or the redirect value is empty
            # TODO: validate that the key corresponds to a Marketing path
            if path != key or not value:
                continue

            try:
                values = {key: value}
                return self.redirect_always(key=key, values=values)
            except Exception as error:  # pylint: disable=broad-except
                LOG.error("The PathRedirectionMiddleware generated an error at: %s%s",
                          request.get_host(),
                          request.get_full_path())
                LOG.error(error)
                return None
        return None

    def login_required(self, request, path, **kwargs):  # pylint: disable=unused-argument
        """
        Action: a user session must exist.
        If it does not, redirect to the login page
        """
        if request.user.is_authenticated:
            return None
        resolved_login_url = configuration_helper.get_dict("FEATURES", {}).get(
            "ednx_custom_login_link", settings.LOGIN_URL)
        return redirect_to_login(path, resolved_login_url, REDIRECT_FIELD_NAME)

    def not_found(self, **kwargs):  # pylint: disable=unused-argument
        """
        Action: return 404 error for anyone
        """
        raise Http404

    def not_found_loggedin(self, request, **kwargs):  # pylint: disable=unused-argument
        """
        Action: return 404 error for users which have a session
        """
        if request.user.is_authenticated:
            raise Http404

    def not_found_loggedout(self, request, **kwargs):  # pylint: disable=unused-argument
        """
        Action: return 404 error for users that don't have a session
        """
        if not request.user.is_authenticated:
            raise Http404

    def redirect_always(self, key, values, **kwargs):  # pylint: disable=unused-argument
        """
        Action: redirect to the given target
        """
        return HttpResponseRedirect(values[key])

    def redirect_loggedin(self, request, key, values, **kwargs):  # pylint: disable=unused-argument
        """
        Action: redirect logged users to the given target
        """
        if request.user.is_authenticated:
            return HttpResponseRedirect(values[key])
        return None

    def redirect_loggedout(self, request, key, values, **kwargs):  # pylint: disable=unused-argument
        """
        Action: redirect external visitors to the given target
        """
        if not request.user.is_authenticated:
            return HttpResponseRedirect(values[key])
        return None


class RedirectionsMiddleware(MiddlewareMixin):
    """
    Middleware for Redirecting microsites to other domains or to error pages
    """

    def process_request(self, request):
        """
        This middleware handles redirections and error pages according to the
        business logic at edunext
        """
        if not settings.FEATURES.get('USE_REDIRECTION_MIDDLEWARE', True):
            return None

        domain = request.META.get('HTTP_HOST', "")

        # First handle the event where a domain has a redirect target
        cache_key = "redirect_cache." + fasthash(domain)
        target = cache.get(cache_key)  # pylint: disable=maybe-no-member

        if not target:
            try:
                target = Redirection.objects.get(domain__iexact=domain)  # pylint: disable=no-member
            except Redirection.DoesNotExist:  # pylint: disable=no-member
                target = '##none'

            cache.set(  # pylint: disable=maybe-no-member
                cache_key, target, 5 * 60
            )

        if target != '##none':
            # If we are already at the target, just return
            if domain == target.target and request.scheme == target.scheme:  # pylint: disable=no-member
                return None

            to_url = f'{target.scheme}://{target.target}{request.path}'

            return HttpResponseRedirect(
                to_url,
                status=target.status,  # pylint: disable=no-member
            )
        return None

    @staticmethod
    @receiver(post_save, sender=Redirection)
    def clear_cache(sender, instance, **kwargs):  # pylint: disable=unused-argument
        """
        Clear the cached template when the model is saved
        """
        cache_key = "redirect_cache." + fasthash(instance.domain)
        cache.delete(cache_key)  # pylint: disable=maybe-no-member


class TPAExceptionMiddleware(ExceptionMiddleware):
    """Middleware to handle exceptions not catched by Social Django"""

    def process_exception(self, request, exception):
        """
        Handle exceptions raised during the authentication process.
        """
        referer_url = request.META.get('HTTP_REFERER', '')
        referer_url = urlparse(referer_url).path

        if referer_url != reverse('signin_user') and request.view_name not in ['auth', 'complete']:
            return super().process_exception(request, exception)

        if isinstance(exception, EoxTenantAuthException):
            new_exception = AuthFailed(
                exception.backend,
                str(exception),
            )
            LOG.error("%s", exception)
            return super().process_exception(request, new_exception)

        if isinstance(exception, IntegrityError):
            backend = getattr(request, 'backend', None)
            new_exception = AuthAlreadyAssociated(
                backend,
                "The given email address is associated with another account",
            )
            LOG.error("%s", exception)
            return super().process_exception(request, new_exception)

        if isinstance(exception, HTTPError):
            backend = getattr(request, 'backend', None)
            new_exception = AuthUnreachableProvider(
                backend,
                "Unable to connect with the external provider",
            )
            LOG.error("%s", exception)
            return super().process_exception(request, new_exception)

        return super().process_exception(request, exception)



"""
Grades definitions.
"""

from importlib import import_module

from django.conf import settings


def get_course_grade_factory():
    """ Gets the CourseGradeFactory object. """

    backend_function = settings.EOX_CORE_GRADES_BACKEND
    backend = import_module(backend_function)

    return backend.get_course_grade_factory()
